c			 Computer Program NLST

c		     Nonlinear Analysis of Space Trusses
c                         January, 2016

c			       by A. Ghali
c			Professor of civil Engineering
c			  The University of Calgary
c			   Calgary, Canada T2N 1N4

c Companion of Ghali,A. and Neville,A. M., Structural Analysis :A Unified
c Classical and Matrix Approach, 7th ed., CRC Press, Spon,Press, 2016
c------------------------------------------------------------------------------
c				 Scope
c    NLST analyses space trusses composed of pin-connected straight prismatic
c  bars and triangular membrane elements. The analysis gives changes in nodal
c  displacements, reactions , axial forces in bars and stresses in membrane
c  elements, accounting for geometric nonlinearity (Chapter 25 of Ghali and
c  Neville. Initially the structure may be subjected to prestressing,
c  temperature changes or nodal forces. Unlike linear analysis, the member
c  end forces in the initial stage must be included in input data. The x, y
c  and z coordinates of the nodes in the input data represent position of the
c  nodes existing before application of new loads for which the changes of
c  displacements and member end forces are required. The loading can be one
c  or cmbination of the following types:
c  (a)Concentrated forces Fx, Fy and Fz applied at the nodes in the
c     direction of the global x,y and z axes .
c  (b)Temperature ,shrinkage ,etc. Input includes axial forces in bars and
c     principal stresses in membrane elements with nodal displacement
c     prevented.
c			      Sign Convention
c    Positive directions of nodal displacements u, v and w are in the
c  positive directions of the global x, y and z axes. The reaction components
c  are determined in the same directions. The axial forces in bars are
c  positive when tensile. The stresses in membrane elements are considered in
c  directions of local axes x* and y* ; these are defined together with 
c  positive stress in Fig. 25-9 of Ghali and Neville.
c
c			  Input Data Instructions
c   The data, in free format , must be in a file named NLST, consisting of:

c - Single line of title ,not exceeding 72 characters.
c - Single line of 6 integers and a real value: number of nodes,number of
c   of bars,number of triangular membrane elements, number of joints with
c   prescribed displacements, number of load stages, maximum number of
c   of iterations for each load stage. The last number is the allowable
c   tolerence,TOLER ;where
c    TOLER=SQRT({g}Transpose {g})/ SQRT({F}Transpose  {F})
c   {g} and {F} are the out-of-balance forces and the nodal forces (total
c   introduced at all stages).

c - Load-stage vector of real multipliers,{NLST-real values}. The
c   external applied forces (load type a) and {Ar} will be introduced
c   in stages, using the multipliers in this vector.

c - Single line of two real values of elasticity modulus for the bars,
c   followed by that for the  membrane elements.

c - A set of lines ; their number equals number of nodes. Each line is
c   composed  of a node number followed by its x, y and z coordinates.

c - A set of lines their number equals number of bars. Each line starts
c   with 3 integers: a member number, its starting node and its end node,
c   followed by its cross-sectional area, a.

c - A set of lines their number equals number of triangular membrane
c   elements bars. Each line starts with 4 integers: element number, its
c   three nodes followed by its thickness.

c - A set of lines their number equals number of joints with prescribed disp-
c   lacements . Each line starts with a node number and three restraint
c   indicators (0 or 1),followed by three real values of prescribed
c   displacements. The restraint indicators 1 and 0 mean free and prescribed
c   displacement component,respectively. When a displacement component is
c   free, a dummy value must be filled in for the prescribed displacement
c   (any real number). For a 2-D system in xy plane allnodes must be included
c   in this set and the indicator 0  and displacement value 0.0 must be
c   entered for w-displacement (Input of  Problem 25.3).

c - A set of any number of lines for loading of type(a). Each line is comp-
c   osed of an integer, a node number,followed by three real values of
c   nodal forces: Fx, Fy and Fz. The last line of the set must start by an
c   greater than the number of nodes  follwed by threee dummy real values.
c   The last line must not be omitted even when there is no loading of
c   type (a).

c - A set of any number of lines for bars with loading of type(b). Each line
c   is composed of an integer, which is a bar number, followed by one real
c   value of axial force , which is caused by temperature, shrikage, etc.
c   when nodal displacements are prevented. The last line of this set must
c   start with an integer greater than the number of bars followed by a
c   dummy real value. This line must not be omitted even when there is no
c   bar with this type of loading.

c - A set of any number of lines for membrane elements with loading  type(b). Each line
c   Each line is composed of an integer, which is element number, followed by
c   three stress values:sigma*x,sigma*y and tau*xy, which are caused by
c   temperature, shrikage, etc. when nodal displacements are prevented. The
c   last line of the set starts  with an integer greater than the number of
c   membrane elements followed by three real values. This line must not be
c   omitted even when there is no elements with this type of loading.

c - A set of any number of lines giving initial axial forces in bars. Each
c   is composed of bar number followed by the initial axial force value
c   (positive when tensile). The last line of the set starts with a number
c   greater than number of bars followed by a dummy real value. This line
c   must not be omitted even when there is no bars with initial force.

c - A set of any number of lines giving initial axial stressess in membrane
c   elements. Each line is composed of element number followed by the initial
c   stresses: segmax, sigmay and tauxy (positive directions defined in Fig.
c   25-9, Ghali and Neville). The last line of the set starts with a number
c   greater than number of elements followed by 3 dummy real values. This
c   line must not be omitted even when there is no elements with initial
c      stresses.

c    The load-stage vector, {STEPS} lists multipliers of the loads of types (a)
c  and (b) are introduced to be applied in the load stages. The prescribed
c  displacements are also introduced in stages using the same multipliers
c  in the load stages. Newton-Raphson's iterations are performed in each stage
c  to eliminate the out-of-balance forces before proceeding to the next stage
c  A support conditions indicator=1 for   any coordinate will mean free
c  displacement in all load stages. The initial forces affect the geometrical
c  stiffness of the members; but the displacements which have occured when
c  the initial prestress is introduced are not the objective of this analysis.
c  Thus the {x} and {y} coordinates of the nodes represent the position of the
c  nodes in the prestressed structure.

c		  Input data examples
c  Example 1:

c  Prob 24-9: space cable net
c  21 24 0 12  1 10   1.e-9	  NJ,NBAR,NME,NSJ,NSTG,NITER,TOLER
c  1.0    {STEPS},{NSTG-load multipliers}
c  10.e6     .0   .0     E for bars, E and Poisson's ratio for membrane
c  1  -1.  -2.	  .25		  Node number, X, Y, Z coordinates
c  2   0.  -2.0   .0
c  3   1.  -2.	 -.25
c  4  -2.  -1.	  .25
c  5  -1.  -1.	 .125
c  6   0.  -1.	  .0
c  7   1.  -1.	-.125
c  8   2.  -1.	-.25
c  9  -2.   .0	  .0
c  10 -1.   .0	  .0
c  11	.0  .0	  .0
c  12  1.   0.	  .0
c  13  2.   0.	  .0
c  14 -2.   1.	 -.25
c  15 -1.   1.	 -.125
c  16  0.   1.	  .0
c  17  1.   1.	  .125
c  18  2.   1.	  .25
c  19 -1.   2.	 -.25
c  20  0.   2.	  .0
c  21  1.   2.	  .25
c  1	4    5	  .001		  Bar no.,1st node ,2nd node, cross-sec. area
c  2	5    6	  .001
c  3	6    7	  .001
c  4	7    8	  .001
c  5	9    10   .001
c  6	10   11   .001
c  7	11   12   .001
c  8	12   13   .001
c  9	14   15   .001
c  10	15   16   .001
c  11	16   17   .001
c  12	17   18   .001
c  13	 1    5   .001
c  14	 5   10   .001
c  15	10   15   .001
c  16	15   19   .001
c  17	 2    6   .001
c  18	 6   11   .001
c  19	11   16   .001
c  20	16   20   .001
c  21	 3    7   .001
c  22	 7   12   .001
c  23	12   17   .001
c  24	17   21   .001
c  1   3*0  3*0.		  Node,restraint indicators,presc. displacements
c  2   3*0  3*0.
c  3   3*0  3*0.
c  4   3*0  3*0.
c  8   3*0  3*0.
c  9   3*0  3*0.
c  13  3*0  3*0.
c  14  3*0  3*0.
c  18  3*0  3*0.
c  19  3*0  3*0.
c  20  3*0  3*0.
c  21  3*0  3*0.
c  5   0.  0.  1.		  NODE, FX, FY, FZ
c  6   0.  0.  1.
c  7   0.  0.  1.
c  10  0.  0.  1.
c  11  0.  0.  1.
c  12  0.  0.  1.
c  15  0.  0.  1.
c  16  0.  0.  1.
c  17  0.  0.  1.
c  100 3*0.		   Dummy: NODE,FX,FY,FZ (end of loads of type a)
c  100	0.  Dummy: Axial force with dipls. restrained:bar,force (end ld. type b)
c  1	30.		   Prestressing axial forces: bar, force
c  2	30.
c  3	30.
c  4	30.
c  5	30.
c  6	30.
c  7	30.
c  8	30.
c  9	30.
c  10	30.
c  11	30.
c  12	30.
c  13	30.
c  14	30.
c  15	30.
c  16	30.
c  17	30.
c  18	30.
c  19	30.
c  20	30.
c  21	30.
c  22	30.
c  23	30.
c  24	30.
c  100	 0. Dummy: Prestress axial forces: bar, force (end of set)
c  100 3*0. Dummy: Membrn. stress displs. restr.: sigmax,sigmay,tau (end of set)
c  100 3*0. Dummy: Membrn. prestress sigmax,sigmay,tau (end of set)

c  Example 2:

c  Use of membrane elements to idealize stretched  cable, Ex. 24-1
c  14  0 12 6 1  6   1.e-6	   NJ,NBAR,NME,NSJ,NSTG,NITER,TOLER
c  1.0	{STEPS},{load multipliers};their number equals no. of ld. stages,NSTG
c  0.  200.e9  0.3		 E for bars, E and Poisson's ratio for membrane
c  1  0.	  .125	  .0	 Node number, X, Y, Z coordinates
c  2  0.	  .0	  .0
c  3  0.16666667  .125	  .0
c  4  0.16666667  .00	  .0
c  5  0.33333333  .125	  .0
c  6  0.33333333  .00	  .0
c  7  0.5	  .125	  .0
c  8  0.5	  .00	  .0
c  9  0.66666667  .125	  .0
c  10 0.66666667  .00	  .0
c  11 0.83333333  .125	  .0
c  12 0.83333333  .00	  .0
c  13 1.0	  .125	  .0
c  14 1.0	  .00	  .0
c  1  1    2	4    .8e-3    Element,nodes i, j and k, respectively,thickness
c  2  4    3	1    .8e-3
c  3  3    4	6    .8e-3
c  4  6    5	3    .8e-3
c  5  5    6	8    .8e-3
c  6  8    7	5    .8e-3
c  7  7    8	10   .8e-3
c  8  10   9	7    .8e-3
c  9  9    10	12   .8e-3
c  10 12   11	9    .8e-3
c  11 11   12	14   .8e-3
c  12 14   13	11   .8e-3
c  1   3*0  3*0.		 Node,restraint indicators,presc. displacements
c  2 0	1  0  3*0.
c  7   1 1 1 0. 0. 0.
c  8   1 1 1 0. 0. 0.
c  13  3*0  3*0.
c  14 0  1  0  3*0.
c  7  0.  0.  13.07e3		 Node, FX, FY, FZ
c  8  0.  0.  13.07e3
c  100	3*0.			 Dummy: Node, Fx, Fy, Fz (end of set)
c  100	0.   Dummy: Axial forces with displs restrained:Bar,force (end of set)
c  100	0.   Dummy: Prestress axial forces: Bar,force (end of set)
c  100 3*0.  Dummy: Membrn. stress displs. restr.:sigmax,sigmay,tau (end of set)
c  1  0.   1.e9      0. 	 Membrane prestress sigmax,sigmay,tau
c  2  0.   1.e9      0.
c  3  0.   1.e9      0.
c  4  0.   1.e9      0.
c  5  0.   1.e9      0.
c  6  0.   1.e9      0.
c  7  0.   1.e9      0.
c  8  0.   1.e9      0.
c  9  0.   1.e9      0.
c  10 0.   1.e9      0.
c  11 0.   1.e9      0.
c  12 0.   1.e9      0.
c  100 3*0.	  Dummy: Membrane prestress sigmax,sigmay,tau (end of set)
c------------------------------------------------------------------------------
c		     Notation and dimensions of arrays
c								     rows  cols.
c  {A}	  cross-section areas					     NBAR
c  {AINIT} axial forces in bars (e. g. due to prestressig)	     NBAR
c  [AR]   axial forces in bars with nodal displacements restrained   NBAR
c  {AUPDAT} updated axial forces in bars			     NBAR
c  {D}	  nodal displacement increments 			     3*NJ
c  {DUPDAT} updated nodal displacements 			     3*NJ
c  E	  modulus of elasticity of bars
c  EM	  modulus of elasticity of membrane elements
c  ET	  thicknesses of membrane elements			     NME
c  [ESTB]  elastic stiffness matrix of a bar in with respect to        6      6
c	  coordinates in global directions
c  [GSB]  gemetric stiffness matrix of abar with respect to coord-     6      6
c	  inates in global directions
c  [ESTM]  elastic stiffness matrix of a membrane element  in	       6      6
c	  to global axes
c  [GSM]  gemetric stiffness matrix of abar with respect to	       6      6
c	  global axes
c  {F}	  nodal restraining forces				    3*NJ
c  {FINIT} initial nodal forces equilibrating {AINIT} and
c	   {SINIT} given as input.				    3*NJ
c  {FUPDAT} part of {F} intoduced in current and earlier	    3*NJ
c	  load stages.
c  {g}	  unbalanced nodal forces				    3*NJ
c  [ISUP] joint restraint indicators : 1 and 0 ,indicating free      NSJ    3
c	  and prescribed displacement respectively
c  {JE}   end nodes of individual bars				     NBAR
c  {JS}   starting node of individual bars			     NBAR
c  NB	  band width ; see Sec. 24-3 of Ghali and Neville
c  NBAR   number of bars
c  NITER  maximum number of iterations in each load stage
c  NJ	  number of joints (nodes)
c  NME	  number of membrane elements
c  NSTG   number of load stages
c  {NS}   list of node numbers where supports are provided or	    NSJ
c	  displacements are prescribed
c  NSJ	  number of supported joints (nodes with prescribed displac-
c	  ements)
c  {MI}   node i (first node)of triangular element		    NME
c  {MJ}   node j (second node)of triangular element		    NME
c  {MK}   node k (third node)of triangular element		    NME
c {OOBF}  out-of-balance forces 				    3*NJ
c  [PD]   total prescribed displacement values introduced in all    NSJ   3
c	  stages
c [PDUPDAT] the part of [PD] intoduced in current and earlier	    NSJ   3
c	  load stages.
c  {R}	  reactions at the three coordinates of supported nodes     3*NSJ
c  {SINIT} initial stresses in membrane elements		    3*NME
c  {SUPDAT} updated stresses in membrane elements		    3*NME
c  SUMOOBF (sum square of out-of-balance forces)**0.5
c  SUMF   (sum square of nodal forces,excluding reactions)**0.5
c  SUMR   (sum square of reaction components)**0.5
c  [S]	  tangent stiffness matrix (upper triangle only)	    3*NJ   NB
c  [SL]   tangent stiffness matrix (triangle below diagonal only)   NB-1  3*NJ-1
c  {STEPS} fractions of load to be introduced in each stage	    NSTG
c  TOLER  tolerence ratio
c  {X}	  x coordinate of nodes before displacements		    NJ
c  {XUPDAT} updated x coordinates (after displacements) 	    NJ
c  {Y}	  y coordinate of nodes before displacements		    NJ
c  {YUPDAT} updated y coordinates (after displacements) 	    NJ
c  {Z}	  Z coordinate of nodes before displacements		    NJ
c  {ZUPDAT} updated Z coordinates (after displacements) 	    NJ
c------------------------------------------------------------------------------
c		 Listing of program NLPF
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXB=20000,MAXNI=2000)
      COMMON/REALV/B(MAXB)
      COMMON/NINTGR/NI(MAXNI)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='NLST.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='NLST.OUT')
      DO 10 I=1,MAXB
10    B(I)=0.0
      DO 20 I=1,MAXNI
20    NI(I)=0
      READ(5,100)TITLE
100   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(3X,A76)
      READ(5,*)NJ,NBAR,NME,NSJ,NSTG,NITER,TOLER
      WRITE(6,300)NJ
300   FORMAT(3X,'Number of joints=',I5)
      WRITE(6,400)NBAR
400   FORMAT(3X,'Number of bars=',I5)
      WRITE(6,450)NME
450   FORMAT(3X,'Number of triangular membrane elements=',I5)
      WRITE(6,500)NSJ
500   FORMAT(3X,'Number of joints with prescribed displacement(s)=',I5)
      WRITE(6,600)NSTG
600   FORMAT(3X,'Number of load stages=',I5)
      WRITE(6,650)NITER
650   FORMAT(3X,'Maximum number of iterations in each load stage=',I3)
      WRITE(6,750)TOLER
750   FORMAT(3X,'Tolerence=',E13.5,/,3X,/,
     +'The tolerence is the allowed error. The error is square root of',
     +/,3X,
     +'sum of squares of out-of-balance forces divided by square root ',
     +/,3X,
     +'of sum of squares of nodal forces {F}. When {F}={0}, the error is
     +',/,3X,
     +'sum of squares of out-of-balance forces divided by square root ',
     +/,3X,
     +'of sum of squares of reaction components.')
      NJ3=3*NJ
      NS3=3*NSJ
c     Elements of {B} at which real arrays start are:
c     {A}:N1   {AINIT}:N2  {AR}:N3  {AUPDAT}:N4  {D}:N5 {DUPDAT}:N6
c     {ET}:N7  {F}:N8	{FUPDAT}:N9   {OOBF}:N10 [PD]:N11
c     [PDUPDAT]:N12 {R}:N13 {RSAVE}:N14 {SINIT}:N15 {SUPDAT}:N16 {STEPS}:N17
c     {X}:N18  {XUPDAT}:N19  {Y}:N20  {YUPDAT}:N21  {Z}:N22  {ZUPDAT}:N23
c     {MI}:N24 {MJ}:N25 {MK}:N26 {SR}:N27
c     [S]:N28 [SL]:N29	{FINIT}:30
      N1=1
      N2=N1+NBAR
      N3=N2+NBAR
      N4=N3+NBAR
      N5=N4+NBAR
      N6=N5+NJ3
      N7=N6+NJ3
      N8=N7+NME
      N9=N8+NJ3
      N10=N9+NJ3
      N11=N10+NJ3
      N12=N11+NS3
      N13=N12+NS3
      N14=N13+NS3
      N15=N14+NS3
      N16=N15+NME*3
      N17=N16+NME*3
      N18=N17+NSTG
      N19=N18+NJ
      N20=N19+NJ
      N21=N20+NJ
      N22=N21+NJ
      N23=N22+NJ
      N24=N23+NJ
      N25=N24+NME
      N26=N25+NME
      N27=N26+NME
      N28=N27+NME*3
c     Elements of {N} at which integer arrays start are:
c     {JS}:K1  {E}:K2  {MI}:K3	{MJ}:K4  {MK}:K5  {NS}:K6  [ISUP]:K7
      K1=1
      K2=K1+NBAR
      K3=K2+NBAR
      K4=K3+NME
      K5=K4+NME
      K6=K5+NME
      K7=K6+NSJ
      NIUSED=K7+NS3-1
c     CALL DATA(X,Y,Z,JS,JE,A,ET,NS,ISUP,PD,NB,NSTG,STEPS,MI,MJ,MK)
      CALL DATA(B(N18),B(N20),B(N22),NI(K1),NI(K2),B(N1),B(N7),NI(K6),
     *NI(K7),B(N11),NB,NSTG,B(N17),NI(K3),NI(K4),NI(K5))
c     Identification of elements of {B} at which real arrays start
c     (continued)
      N29=N28+NJ3*NB
      N30=N29+NB*(NJ3-1)
      NBUSED=N30+NJ3-1
      WRITE(6,755)NBUSED,NIUSED
755   FORMAT(/,3X
     +	'This problem requires a vector of',I5,' real values.',
     +/,3X,'It also requies a vector of',I4,' integers.')
c     CALL LOAD(JS,JE,X,Y,Z,F,AR,SR,NJ3,AINIT,FINIT,NS,RSAVE,MI,MJ,MK,
c    + SINIT,ET)
      CALL LOAD(NI(K1),NI(K2),B(N18),B(N20),B(N22),B(N8),B(N3),B(N27)
     + ,NJ3,B(N2),B(N30),NI(K6),B(N14),NI(K3),NI(K4),NI(K5),
     + B(N15),B(N7))
c     CALL STARTX_Y_Z(XUPDAT,YUPDAT,ZUPDAT,X,Y,Z)
      CALL STARTX_Y_Z(B(N19),B(N21),B(N23),B(N18),B(N20),B(N22))
c     CALL STARTA(AUPDAT,AINIT)
      CALL STARTA(B(N4),B(N2))
c     CALL STARTST(SUPDAT,SINIT)
      CALL STARTST(B(N16),B(N15))
      DO 999 NSTEP=1,NSTG
      WRITE(6,150)NSTEP
150   FORMAT(3X,'Load stage number ',I2)
c     CALL ADDAR(AUPDAT,AR,STEPS,NSTEP,NSTG,FACTOR)
      CALL ADDAR(B(N4),B(N3),B(N17),NSTEP,NSTG,FACTOR)
      WRITE(6,145)FACTOR
145   FORMAT(3X,'Factor representing load level reached at this stage=',
     +E15.5)
c     CALL ADDSR(SUPDAT,SR,STEPS,NSTEP,NSTG)
      CALL ADDSR(B(N16),B(N27),B(N17),NSTEP,NSTG)
c     CALL UPDATEF(NJ3,F,FUPDAT,STEPS,NSTG,NSTEP,PD,PDUPDAT,SUMF,
c    + NS,ISUP)
      CALL UPDATEF(NJ3,B(N8),B(N9),B(N17),NSTG,NSTEP,B(N11),B(N12),SUMF,
     +	NI(K6),NI(K7))
      ITER=0
800   CONTINUE
c     CALL UPDATEX_Y_Z(XUPDAT,YUPDAT,ZUPDAT,D,NJ3)

c     Updating coordinates is moved below in Dec. 2000 (2 statements above 995).
c     CALL UPDATEX_Y_Z(B(N19),B(N21),B(N23),B(N5),NJ3)
c     CALL OOBFS(JS,JE,XUPDAT,YUPDAT,ZUPDAT,FUPDAT,AUPDAT,SUPDAT,
c    +NJ3,OOBF,SUMOOBF, NS,ISUP,SUMR,R,FINIT,RSAVE,NSTEP,STEPS,NSTG,AR,
c    +SR,MI,MJ,MK,ET,X,Y,Z)
      CALL OOBFS(NI(K1),NI(K2),B(N19),B(N21),B(N23),B(N9),B(N4),B(N16),
     +NJ3,B(N10),SUMOOBF,NI(K6),NI(K7),SUMR,B(N13),B(N30),B(N14),NSTEP,
     +B(N17),NSTG,B(N3),B(N27),NI(K3),NI(K4),NI(K5),B(N7),
     +B(N18),B(N20),B(N22))
      IF(ITER.LT.1) GO TO 11
      IF(SUMF.EQ.0.0)ERROR=SUMOOBF/SUMR
      IF(SUMF.NE.0.0)ERROR=SUMOOBF/SUMF
      IF(ERROR.LE.TOLER)GO TO 995
11    CONTINUE
      ITER=ITER+1
      IF(ITER.GT.NITER)GO TO 995
c     WRITE(6,350)ITER
350   FORMAT(3X,'Iteration number ',I2)
c     CALL STIF(JS,JE,A,XUPDAT,YUPDAT,ZUPDAT,S,NJ3,NB,AUPDAT,SUPDAT,
c    + X,Y,Z,ET,MI,MJ,MK,SL)
      CALL STIF(NI(K1),NI(K2),B(N1),B(N19),B(N21),B(N23),B(N28),
     *NJ3,NB,B(N4),B(N16),B(N18),B(N20),B(N22),B(N7),
     +NI(K3),NI(K4),NI(K5),B(N29))
c     CALL SUPTPF(S,OOBF,NS,ISUP,PDUPDAT,NJ3,NB,ITER)
      CALL SUPTPF(B(N28),B(N10),NI(K6),NI(K7),B(N12),NJ3,NB,
     +	ITER)
c     CALL SOLVE(S,SL,NB,D,OOBF,NJ3,1)
      CALL SOLVE(B(N28),B(N29),NB,B(N5),B(N10),NJ3,1)
c     CALL UPDATEA(D,XUPDAT,YUPDAT,ZUPDAT,JS,JE,A,AUPDAT,NJ3,
c     + X,Y,Z,AINIT)
      CALL UPDATEA(B(N5),B(N19),B(N21),b(N23),NI(K1),NI(K2),B(N1),B(N4),
     +	 NJ3, B(N18),B(N20),B(N22),B(N2))
c     CALL UPDATES(D,XUPDAT,YUPDAT,ZUPDAT,MI,MJ,MK,SUPDAT,NJ3,
c    +	X,Y,Z,SINIT)
      CALL UPDATES(B(N5),B(N19),B(N21),B(N23),NI(K3),NI(K4),NI(K5),
     +B(N16),NJ3,B(N18),B(N20),B(N22),B(N15))
c     CALL UPDATED(DUPDAT,D,NJ3)
      CALL UPDATED(B(N6),B(N5),NJ3 )

c     CALL UPDATEX_Y_Z(XUPDAT,YUPDAT,ZUPDAT,D,NJ3)
      CALL UPDATEX_Y_Z(B(N19),B(N21),B(N23),B(N5),NJ3)

      GO TO 800
995   CONTINUE
c     CALL RESULTS(DUPDAT,NS,R,AUPDAT,SUPDAT,NJ3,ITER,ERROR)
      CALL RESULTS(B(N6),NI(K6),B(N13),B(N4),B(N16),NJ3,ITER,ERROR)
999   CONTINUE
      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATA(X,Y,Z,
     + JS,JE,A,ET,NS,ISUP,PD,NB,NSTG,STEPS,MI,MJ,MK)
c     Reading data for plane frame (excluding loads)and calculation of
c     band width.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION X(NJ),Y(NJ),Z(NJ),JS(NBAR),JE(NBAR),A(NBAR)
      DIMENSION MI(NME),MJ(NME),MK(NME),ET(NME)
      DIMENSION NS(NSJ),ISUP(NSJ,3),PD(NSJ,3),STEPS(NSTG)
c     Read the load fractions to be introduced in the load stages.
      READ(5,*)(STEPS(I),I=1,NSTG)
      WRITE(6,640)(STEPS(I),I=1,NSTG)
640   FORMAT(/,3X,'Load fractions in different stages:',/,3X,
     +10(F6.4,1X))
      READ(5,*)EB,EM,PRM
      WRITE(6,200)EB,EM,PRM
200   FORMAT(/,3X,'Elasticity modulus for bars=',E13.5,
     +	     /,3X,'Elasticity modulus for membrane elements=',E13.5,
     +	     /,3X,'Poisson s ratio for membrane elements=',E13.5)
      WRITE(6,300)
300   FORMAT(/,3X,'Nodal coordinates')
      WRITE(6,400)
400   FORMAT(5X,'Node',5x,'x',9X,'y',9X,'z')
      DO 10 I=1,NJ
      READ(5,*)J,X(J),Y(J),Z(J)
10    WRITE(6,500)J,X(J),Y(J),Z(J)
500   FORMAT(3X,I5,3F10.3)
      WRITE(6,610)
610   FORMAT(/,3X,'Element information: bars')
      WRITE(6,710)
710   FORMAT(3X,'Element',2X,'1st node',3X,'2nd node',6x,'a')
      DO 20 I=1,NBAR
      READ(5,*)J,JS(J),JE(J),A(J)
c     Bar no.,starting node,end node,cross-sectional area
20    WRITE(6,800)J,JS(J),JE(J),A(J)
800   FORMAT(3X,I5,4X,I5,5X,I5,3X,E13.5)
      WRITE(6,600)
600   FORMAT(/,3X,'Element information: triangular membrane elements')
      WRITE(6,700)
700   FORMAT(3X,'Element',2X,'node i',4X,'node j',4X,'node k',4x,
     +'thickness')
      DO 25 I=1,NME
      READ(5,*)J,MI(J),MJ(J),MK(J),ET(J)
c     Elementr no.,node i,node j,node k, thickness
25    WRITE(6,810)J,MI(J),MJ(J),MK(J),ET(J)
810   FORMAT(3X,I5,4X,I5,5X,I5,5X,I5,3X,E13.5)
      WRITE(6,900)
900   FORMAT(/,3X,'Support conditions')
      WRITE(6,1000)
1000  FORMAT(5X,'Node',5X,'Restraint indicators',7X,'Prescribed displace
     *ments')
c     Restraint indicators are 1 or 0, meaning free or prescribed disp-
c     lacement,respectively.
      WRITE(6,1100)
1100  FORMAT(16x,'u',6x,'v',6X,'w',8x,'u',13x,'v',13x,'w')
1200  FORMAT(3X,I5,4X,I5,2X,I5,2X,I5,3(2X,E13.5))
      DO 30 J=1,NSJ
      READ(5,*)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),PD(J,
     *3)
30    WRITE(6,1200)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),
     *PD(J,3)
c     Determine band width by Eq. 22-1 of Ghali and Neville.
      NBI=0
      DO 50 I=1,NBAR
      N=JS(I)-JE(I)
      IF(N.GE.0) GO TO 40
      N=-N
40    CONTINUE
      IF(N.GT.NBI)NBI=N
50    CONTINUE
      DO 55 I=1,NME
      N1=ABS(MJ(I)-MI(I))
      N2=ABS(MK(I)-MI(I))
      IF(N1.GE.N2)N=N1
      IF(N1.LT.N2)N=N2
      IF(N.GT.NBI)NBI=N
55    CONTINUE
      NB=3*(NBI+1)
      RETURN
      END

      SUBROUTINE LOAD(JS,JE,X,Y,Z,F,AR,SR,NJ3,AINIT,FINIT,NS,RSAVE,
     + MI,MJ,MK,SINIT,ET)
c     Generation of vectors of restraining forces,{F}
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION JS(NBAR),JE(NBAR),X(NJ),Y(NJ),Z(NJ),F(NJ3)
      DIMENSION AR(NBAR),AINIT(NBAR),FINIT(3*NJ),NS(NSJ),RSAVE(NSJ,3)
      DIMENSION MI(NME),MJ(NME),MK(NME),SR(3*NME),SINIT(3*NME)
      DIMENSION T(3,3),ET(NME),FSTAR(9),FT(9)
c     Forces applied at the nodes.Note that the last line of this set
c     of input lines must start with an integer > number of nodes;
c     the line must be completed by 3 real numbers.
      WRITE(6,100)
100   FORMAT(/,3X,'Forces applied at the nodes')
      WRITE(6,200)
200   FORMAT(3X,'Node',12X,'Fx',12X,'Fy',12X,'Fz')
20    CONTINUE
      READ(5,*)NODE,FX,FY,FZ
      WRITE(6,300)NODE,FX,FY,FZ
300   FORMAT(3X,I5,6X,3(1X,E13.5))
      IF(NODE.GT.NJ)GO TO 30
      N1=3*NODE-2
      N2=N1+1
      N3=N2+1
      F(N1)=F(N1)-FX
      F(N2)=F(N2)-FY
      F(N3)=F(N3)-FZ
c     Save forces applied on the supported nodes to be used in calculation of
c     reactions; see subroutine OOBS.
      DO 21 L=1,NSJ
      IF(NODE.EQ.NS(L))RSAVE(L,1)=-FX
      IF(NODE.EQ.NS(L))RSAVE(L,2)=-FY
      IF(NODE.EQ.NS(L))RSAVE(L,3)=-FZ
21    CONTINUE
      GO TO 20
30    CONTINUE
c     Fixed-end forces(given in data)due to loading away from the nodes
c     due to temperature,etc..Note that the last line in this set of
c     data lines must start with an integer greater than NBAR;the
c     line must be completed by  three real numbers. The first real number
c     is the axial force (positive when tensile), the second and third real
c     values are the fixed-end moments (positive when clockwise).
c     The positive directions of the end forces are shown in Fig.22-2 of
      WRITE(6,400)
400   FORMAT(/,3X,
     *'Axial forces in bars with nodal displacement restrained')
      WRITE(6,500)
500   FORMAT(3X,'Member',2X,'Axial force')
40    CONTINUE
      READ(5,*)K,Axial
      WRITE(6,600)K,Axial
      IF(K.GT.NBAR)GO TO 50
      AR(K)=Axial
600   FORMAT(3X,I5,1X,E10.4)
c     Transform the fixed-end forces into global directions and add to
c     the existing vectors of restraining forces.To do this,identify the
c     node and coordinate numbers at the member ends and calculate the
c     cosine and sine the angle that the member makes with the global
c     x axis.
      ALX=X(JE(K))-X(JS(K))
      ALY=Y(JE(K))-Y(JS(K))
      ALZ=Z(JE(K))-Z(JS(K))
      AL=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      CX=ALX/AL
      CY=ALY/AL
      CZ=ALZ/AL
      A1=-AR(K)
      A2=AR(K)
      N1=3*JS(K)-2
      N2=N1+1
      N3=N1+2
      F(N1)=F(N1)+A1*CX
      F(N2)=F(N2)+A1*CY
      F(N3)=F(N3)+A1*CZ
      N1=3*JE(K)-2
      N2=N1+1
      N3=N1+2
      F(N1)=F(N1)+A2*CX
      F(N2)=F(N2)+A2*CY
      F(N3)=F(N3)+A2*CZ
      GO TO 40
50    CONTINUE
c     Read initial axial forces in bars (caused for example by prestress). The
c     displacement caused by the prestressing are not determined by this
c     analysis. For each prestressed member, read one line composed of an
c     integer (member number) and one real value. The  real value is
c     the axial force (positive when tensile).
c     The last line in this set of data lines must start with an integer
c     greater than NBAR;the line must be completed by  one real value.
c     Generate {FINIT}=initial nodal forces which must exist to equilibrate
c     the given initial member end forces. The displacements due to the
c     initial forces are not of concern of this computer program.
      WRITE(6,450)
450   FORMAT(/,3X,'Axial forces in bars due to initial prestressing')
      WRITE(6,500)
60    CONTINUE
      READ(5,*)K,Axial
      WRITE(6,600)K,Axial
      IF(K.GT.NBAR)GO TO 70
      AINIT(K)=Axial
      A1=-Axial
      A2=Axial
      ALX=X(JE(K))-X(JS(K))
      ALY=Y(JE(K))-Y(JS(K))
      ALZ=Z(JE(K))-Z(JS(K))
      AL=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      CX=ALX/AL
      CY=ALY/AL
      CZ=ALZ/AL
      FINIT(3*JS(K)-2)=FINIT(3*JS(K)-2)+A1*CX
      FINIT(3*JS(K)-1)=FINIT(3*JS(K)-1)+A1*CY
      FINIT(3*JS(K))=FINIT(3*JS(K))+A1*CZ
      FINIT(3*JE(K)-2)=FINIT(3*JE(K)-2)+A2*CX
      FINIT(3*JE(K)-1)=FINIT(3*JE(K)-1)+A2*CY
      FINIT(3*JE(K))=FINIT(3*JE(K))+A2*CZ
      GO TO 60
70    CONTINUE
      WRITE(6,670)
670   FORMAT(/,3X,
     +'Sresses in membrane elements with nodal displacements prevented')
      WRITE(6,680)
680   FORMAT(3X,'Element',2X,'Sigmax',7X,'Sigmay',10X,
     *'Tauxy')
64    CONTINUE
      READ(5,*)K,Sigmax,Sigmay,Tau
      WRITE(6,690)K,Sigmax,Sigmay,Tau
      IF(K.GT.NME)GO TO 75
      SR(3*K-2)=Sigmax
      SR(3*K-1)=Sigmay
      SR(3*K)=Tau
690   FORMAT(3X,I5,1X,3(E10.4,5X))
      XI=X(MI(K))
      YI=Y(MI(K))
      ZI=Z(MI(K))
      XJ=X(MJ(K))
      YJ=Y(MJ(K))
      ZJ=Z(MJ(K))
      XK=X(MK(K))
      YK=Y(MK(K))
      ZK=Z(MK(K))
c     Direction cosines of local axes of element
      CALL COSLOCAL(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,ALIJ,ALRK,ALIR)
c     Find coordinates of element corners with respect to local axes.
      CALL TRANSFCOR(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,
     + XSI,YSI,ZSI,XSJ,YSJ,ZSJ,XSK,YSK,ZSK)
c     Element nodal forces in local directions corresponding to stresses {SR}
c     (Eq. 24-72 of Ghali and Neville).
      FSTAR(1)=(-(YSK-YSJ)*Sigmax-(XSJ-XSK)*Tau)*ET(K)/2.
      FSTAR(2)=((XSK-XSJ)*Sigmay-(YSK-YSJ)*Tau)*ET(K)/2.
      FSTAR(3)=0.
      FSTAR(4)=(-(YSI-YSK)*Sigmax-(XSK-XSI)*Tau)*ET(K)/2.
      FSTAR(5)=((XSI-XSK)*Sigmay-(YSI-YSK)*Tau)*ET(K)/2.
      FSTAR(6)=0.
      FSTAR(7)=(-(XSI-XSJ)*Tau)*ET(K)/2.
      FSTAR(8)=((XSJ-XSI)*Sigmay)*ET(K)/2.
      FSTAR(9)=0.
c     Transform element forces from local to global coordinates.
      CALL TRANSFMF(FSTAR,T,FT)
      N1=3*MI(K)-2
      N2=N1+1
      N3=N1+2
      F(N1)=F(N1)+FT(1)
      F(N2)=F(N2)+FT(2)
      F(N3)=F(N3)+FT(3)
      N1=3*MJ(K)-2
      N2=N1+1
      N3=N1+2
      F(N1)=F(N1)+FT(4)
      F(N2)=F(N2)+FT(5)
      F(N3)=F(N3)+FT(6)
      N1=3*MK(K)-2
      N2=N1+1
      N3=N1+2
      F(N1)=F(N1)+FT(7)
      F(N2)=F(N2)+FT(8)
      F(N3)=F(N3)+FT(9)
      GO TO 64
75    CONTINUE
c     Read initial stresses in membrane elements.
      WRITE(6,750)
750   FORMAT(/,3X,
     +'Stresses in membrane elements due to initial prestressing')
      WRITE(6,680)
74    CONTINUE
      READ(5,*)K,Sigmax,Sigmay,Tau
      WRITE(6,690)K,Sigmax,Sigmay,Tau
      IF(K.GT.NME)GO TO 85
      SINIT(3*K-2)=Sigmax
      SINIT(3*K-1)=Sigmay
      SINIT(3*K)=Tau
      XI=X(MI(K))
      YI=Y(MI(K))
      ZI=Z(MI(K))
      XJ=X(MJ(K))
      YJ=Y(MJ(K))
      ZJ=Z(MJ(K))
      XK=X(MK(K))
      YK=Y(MK(K))
      ZK=Z(MK(K))
c     Direction cosines of local axes of element
      CALL COSLOCAL(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,ALIJ,ALRK,ALIR)
c     Find coordinates of element corners with respect to local axes.
      CALL TRANSFCOR(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,
     + XSI,YSI,ZSI,XSJ,YSJ,ZSJ,XSK,YSK,ZSK)
c     Element nodal forces in local directions corresponding to stresses {SR}
c     (Eq. 24-72 of Ghali and Neville).
      FSTAR(1)=(-(YSK-YSJ)*Sigmax-(XSJ-XSK)*Tau)*ET(K)/2.
      FSTAR(2)=((XSK-XSJ)*Sigmay-(YSK-YSJ)*Tau)*ET(K)/2.
      FSTAR(3)=0.
      FSTAR(4)=(-(YSI-YSK)*Sigmax-(XSK-XSI)*Tau)*ET(K)/2.
      FSTAR(5)=((XSI-XSK)*Sigmay-(YSI-YSK)*Tau)*ET(K)/2.
      FSTAR(6)=0.
      FSTAR(7)=(-(XSI-XSJ)*Tau)*ET(K)/2.
      FSTAR(8)=((XSJ-XSI)*Sigmay)*ET(K)/2.
      FSTAR(9)=0.
c     Transform element forces from local to global coordinates.
      CALL TRANSFMF(FSTAR,T,FT)
      N1=3*MI(K)-2
      N2=N1+1
      N3=N1+2
      FINIT(N1)=FINIT(N1)+FT(1)
      FINIT(N2)=FINIT(N2)+FT(2)
      FINIT(N3)=FINIT(N3)+FT(3)
      N1=3*MJ(K)-2
      N2=N1+1
      N3=N1+2
      FINIT(N1)=FINIT(N1)+FT(4)
      FINIT(N2)=FINIT(N2)+FT(5)
      FINIT(N3)=FINIT(N3)+FT(6)
      N1=3*MK(K)-2
      N2=N1+1
      N3=N1+2
      FINIT(N1)=FINIT(N1)+FT(7)
      FINIT(N2)=FINIT(N2)+FT(8)
      FINIT(N3)=FINIT(N3)+FT(9)
      GO TO 74
85    CONTINUE
      WRITE(6,700)
700   FORMAT(/,3X,'Initial nodal forces necessary for equilibrium of',/
     + ,3X,'the initial prestressing forces given above')
      WRITE(6,200)
      DO 95 I=1,NJ
95    WRITE(6,300)I,FINIT(3*I-2),FINIT(3*I-1),FINIT(3*I)
      WRITE(6,149)
149   FORMAT(3X,'------------------------------------------',
     +'---------------------------------------')
      RETURN
      END

      SUBROUTINE COSLOCAL(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,ALIJ,ALRK,ALIR)
c     Direction cosines of local axes of membrane triangular element
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION T(3,3)
      ALXIJ=XJ-XI
      ALYIJ=YJ-YI
      ALZIJ=ZJ-ZI
      ALIJ=DSQRT(ALXIJ*ALXIJ+ALYIJ*ALYIJ+ALZIJ*ALZIJ)
      T(1,1)=ALXIJ/ALIJ
      T(1,2)=ALYIJ/ALIJ
      T(1,3)=ALZIJ/ALIJ
      ALXIK=XK-XI
      ALYIK=YK-YI
      ALZIK=ZK-ZI
      ALIK=DSQRT(ALXIK*ALXIK+ALYIK*ALYIK+ALZIK*ALZIK)
      CXIK=ALXIK/ALIK
      CYIK=ALYIK/ALIK
      CZIK=ALZIK/ALIK
c     Determine point r intersection of a perpindicular from on ij.
      ALIR=ALIK*(T(1,1)*CXIK+T(1,2)*CYIK+T(1,3)*CZIK)
      XR=XI*(1.-ALIR/ALIJ)+XJ*(ALIR/ALIJ)
      YR=YI*(1.-ALIR/ALIJ)+YJ*(ALIR/ALIJ)
      ZR=ZI*(1.-ALIR/ALIJ)+ZJ*(ALIR/ALIJ)
      ALXRK=XK-XR
      ALYRK=YK-YR
      ALZRK=ZK-ZR
      ALRK=DSQRT(ALXRK*ALXRK+ALYRK*ALYRK+ALZRK*ALZRK)
      T(2,1)=ALXRK/ALRK
      T(2,2)=ALYRK/ALRK
      T(2,3)=ALZRK/ALRK
c     Cross product of vectors ij and rk.
      T(3,1)=T(1,2)*T(2,3)-T(1,3)*T(2,2)
      T(3,2)=T(1,3)*T(2,1)-T(1,1)*T(2,3)
      T(3,3)=T(1,1)*T(2,2)-T(1,2)*T(2,1)
      RETURN
      END

      SUBROUTINE TRANSFCOR(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,
     + XSI,YSI,ZSI,XSJ,YSJ,ZSJ,XSK,YSK,ZSK)
c     Find coordinates of element corners with respect to local axes.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION T(3,3)
      XSI=T(1,1)*XI+T(1,2)*YI+T(1,3)*ZI
      YSI=T(2,1)*XI+T(2,2)*YI+T(2,3)*ZI
      ZSI=T(3,1)*XI+T(3,2)*YI+T(3,3)*ZI
      XSJ=T(1,1)*XJ+T(1,2)*YJ+T(1,3)*ZJ
      YSJ=T(2,1)*XJ+T(2,2)*YJ+T(2,3)*ZJ
      ZSJ=T(3,1)*XJ+T(3,2)*YJ+T(3,3)*ZJ
      XSK=T(1,1)*XK+T(1,2)*YK+T(1,3)*ZK
      YSK=T(2,1)*XK+T(2,2)*YK+T(2,3)*ZK
      ZSK=T(3,1)*XK+T(3,2)*YK+T(3,3)*ZK
      RETURN
      END

      SUBROUTINE TRANSFMF(FSTAR,T,FT)
c     Transform element forces from local to global
c     {FSTAR}={{Fx*,Fy*,Fz*}i,{Fx*,Fy*,Fz*}j,{Fx*,Fy*,Fz*}k}
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION T(3,3), FT(9), FSTAR(9)
      DO 10 L=1,9
10    FT(L)=0.
      DO 20 L=1,3
      DO 20 I=1,3
      DO 20 J=1,3
      FT(3*(L-1)+I)=FT(3*(L-1)+I)+T(J,I)*FSTAR(3*(L-1)+J)
20    CONTINUE
      RETURN
      END

      SUBROUTINE STARTX_Y_Z(XUPDAT,YUPDAT,ZUPDAT,X,Y,Z)
c     Start the analysis by setting the updated coordinates of the nodes
c     equal to the given data values.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION X(NJ),Y(NJ),Z(NJ),XUPDAT(NJ),YUPDAT(NJ),ZUPDAT(NJ)
      DO 6 I=1,NJ
      XUPDAT(I)=X(I)
      YUPDAT(I)=Y(I)
      ZUPDAT(I)=Z(I)
6     CONTINUE
      RETURN
      END

      SUBROUTINE STARTA(AUPDAT,AINIT)
c     Set the axial forces in the bars equal to the initial presressing values.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION AUPDAT(NBAR),AINIT(NBAR)
      DO 5 I=1,NBAR
      AUPDAT(I)=AINIT(I)
5     CONTINUE
      RETURN
      END

      SUBROUTINE ADDAR(AUPDAT,AR,STEPS,NSTEP,NSTG,FACTOR)
c     Add the fraction of the restraining axial forces in bars {Ar} introduced
c     in the current load stage to the axial forces in bars.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION AUPDAT(NBAR),AR(NBAR),STEPS(NSTG)
      COOF=STEPS(NSTEP)
      DO 5 I=1,NBAR
      AUPDAT(I)=AUPDAT(I)+COOF*AR(I)
5     CONTINUE
      FACTOR=0.
      DO 6 I=1,NSTEP
6     FACTOR=FACTOR+STEPS(I)
      RETURN
      END

      SUBROUTINE STARTST(SUPDAT,SINIT)
c     Set the stresses in membrane elements equal to the initial presressing
c     values.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION SUPDAT(NME*3),SINIT(NME*3)
      DO 5 I=1,NME*3
      SUPDAT(I)=SINIT(I)
5     CONTINUE
      RETURN
      END

      SUBROUTINE ADDSR(SUPDAT,SR,STEPS,NSTEP,NSTG)
c     Add the fraction of the restraining stresses in membrane elements {SR}
c     introduced in the current load stage to the element stresses.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION SUPDAT(NME*3),SR(NME*3),STEPS(NSTG)
      COOF=STEPS(NSTEP)
      DO 5 I=1,NME*3
      SUPDAT(I)=SUPDAT(I)+COOF*SR(I)
5     CONTINUE
      RETURN
      END

      SUBROUTINE UPDATEF(NJ3,F,FUPDAT,STEPS,NSTG,NSTEP,PD,PDUPDAT,SUMF,
     + NS,ISUP)
c     Calculation of the nodal forces after applying new load stage.
c     Also, determine the prescribed displacement after introducing
c     afraction in the new load stage.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION F(NJ3),FUPDAT(NJ3),STEPS(NSTG),PD(NSJ,3),PDUPDAT(NSJ,3)
      DIMENSION NS(NSJ),ISUP(NSJ,3)
      COOF=0.
      DO 5 K=1,NSTEP
5     COOF=COOF+STEPS(K)
      DO 10 I=1,NJ3
10    FUPDAT(I)=COOF*F(I)
      DO 15 I=1,NSJ
      DO 15 J=1,3
15    PDUPDAT(I,J)=STEPS(NSTEP)*PD(I,J)
      IF(NSTEP.EQ.1)GO TO 16
      COOF=COOF-STEPS(NSTEP)
16    CONTINUE
c     Calculate sum of squares of nodal forces before the current load stage.
c     Exclude forces at coordinates with prescribed displacement.
      SUMF=0.
      DO 20 I=1,NJ
      DO 17 K=1,NSJ
17    IF(I.EQ.NS(K))GO TO 18
      SUMF=SUMF+(COOF*F(3*I-2))**2
      SUMF=SUMF+(COOF*F(3*I-1))**2
      SUMF=SUMF+(COOF*F(3*I))**2
      GO TO 19
18    CONTINUE
      IF(ISUP(K,1).EQ.1)SUMF=SUMF+(COOF*F(3*I-2))**2
      IF(ISUP(K,2).EQ.1)SUMF=SUMF+(COOF*F(3*I-1))**2
      IF(ISUP(K,3).EQ.1)SUMF=SUMF+(COOF*F(3*I))**2
19    CONTINUE
20    CONTINUE
      SUMF=DSQRT(SUMF)
      RETURN
      END

      SUBROUTINE UPDATEX_Y_Z(XUPDAT,YUPDAT,ZUPDAT,D,NJ3)
c     Update the coordinates of nodes.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION D(NJ3),XUPDAT(NJ),YUPDAT(NJ),ZUPDAT(NJ)
      DO 10 I=1,NJ
      XUPDAT(I)=XUPDAT(I)+D(3*I-2)
      YUPDAT(I)=YUPDAT(I)+D(3*I-1)
10    ZUPDAT(I)=ZUPDAT(I)+D(3*I)
      RETURN
      END

      SUBROUTINE OOBFS(JS,JE,XUPDAT,YUPDAT,ZUPDAT,FUPDAT,AUPDAT,SUPDAT,
     +NJ3,OOBF,SUMOOBF, NS,ISUP,SUMR,R,FINIT,RSAVE,NSTEP,STEPS,NSTG,AR,
     +SR,MI,MJ,MK,ET,X,Y,Z)
c     Calculation of out of balance forces at the start of an iteration cycle.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION OOBF(NJ3),FUPDAT(NJ3),AUPDAT(NBAR),SUPDAT(NME*3)
      DIMENSION XUPDAT(NJ),YUPDAT(NJ),ZUPDAT(NJ),R(NSJ,3),FINIT(3*NJ)
      DIMENSION JS(NBAR),JE(NBAR),NS(NSJ),ISUP(NSJ,3)
      DIMENSION RSAVE(NSJ,3),STEPS(NSTG),AR(NBAR),SR(NME*3)
      DIMENSION T(3,3),TIN(3,3),FSTAR(9),FT(9)
      DIMENSION MI(NME),MJ(NME),MK(NME),ET(NME),X(NJ),Y(NJ),Z(NJ)
      COOF=0.
      DO 2  I=1,NSTEP
2     COOF=COOF+STEPS(I)
      DO 5 I=1,NJ3
5     OOBF(I)=FUPDAT(I)
      DO 10 L=1,NBAR
      ALX=XUPDAT(JE(L))-XUPDAT(JS(L))
      ALY=YUPDAT(JE(L))-YUPDAT(JS(L))
      ALZ=ZUPDAT(JE(L))-ZUPDAT(JS(L))
      AL=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      CX=ALX/AL
      CY=ALY/AL
      CZ=ALZ/AL
      A2=AUPDAT(L)
      A1=-A2
c     Add the components -in the global directions- of the updated bar end
c     forces to the vector of out-of-balance forces.
      N1=3*JS(L)-2
      N2=N1+1
      N3=N1+2
      OOBF(N1)=OOBF(N1)+A1*CX-FINIT(N1)
      OOBF(N2)=OOBF(N2)+A1*CY-FINIT(N2)
      OOBF(N3)=OOBF(N3)+A1*CZ-FINIT(N3)
      N1=3*JE(L)-2
      N2=N1+1
      N3=N1+2
      OOBF(N1)=OOBF(N1)+A2*CX-FINIT(N1)
      OOBF(N2)=OOBF(N2)+A2*CY-FINIT(N2)
      OOBF(N3)=OOBF(N3)+A2*CZ-FINIT(N3)
c     Nodal forces due to {Ar} are included in {FUPDAT} and in {AUPDAT} which
c     are used above to generate {OOBF}. Deduct below the nodal forces due to
c     {Ar}.
      A2=COOF*AR(L)
      A1=-A2
      OOBF(3*JS(L)-2)=OOBF(3*JS(L)-2)-(A1*CX)
      OOBF(3*JS(L)-1)=OOBF(3*JS(L)-1)-(A1*CY)
      OOBF(3*JS(L))=OOBF(3*JS(L))-(A1*CZ)
      OOBF(3*JE(L)-2)=OOBF(3*JE(L)-2)-(A2*CX)
      OOBF(3*JE(L)-1)=OOBF(3*JE(L)-1)-(A2*CY)
      OOBF(3*JE(L))=OOBF(3*JE(L))-(A2*CZ)
10    CONTINUE
      DO 11 K=1,NME
      Sigmax=SUPDAT(3*K-2)
      Sigmay=SUPDAT(3*K-1)
      Tau=SUPDAT(3*K)
      XI=XUPDAT(MI(K))
      YI=YUPDAT(MI(K))
      ZI=ZUPDAT(MI(K))
      XJ=XUPDAT(MJ(K))
      YJ=YUPDAT(MJ(K))
      ZJ=ZUPDAT(MJ(K))
      XK=XUPDAT(MK(K))
      YK=YUPDAT(MK(K))
      ZK=ZUPDAT(MK(K))
c     Direction cosines of local axes of element
      CALL COSLOCAL(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,ALIJ,ALRK,ALIR)
c     Find coordinates of element corners with respect to local axes.
      CALL TRANSFCOR(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,
     + XSI,YSI,ZSI,XSJ,YSJ,ZSJ,XSK,YSK,ZSK)
c     Element nodal forces in local directions corresponding to stresses
c     {SUPDAT} (Eq. 24-72 of Ghali and Neville).
      FSTAR(1)=(-(YSK-YSJ)*Sigmax-(XSJ-XSK)*Tau)*ET(K)/2.
      FSTAR(2)=((XSK-XSJ)*Sigmay-(YSK-YSJ)*Tau)*ET(K)/2.
      FSTAR(3)=0.
      FSTAR(4)=(-(YSI-YSK)*Sigmax-(XSK-XSI)*Tau)*ET(K)/2.
      FSTAR(5)=((XSI-XSK)*Sigmay-(YSI-YSK)*Tau)*ET(K)/2.
      FSTAR(6)=0.
      FSTAR(7)=(-(XSI-XSJ)*Tau)*ET(K)/2.
      FSTAR(8)=((XSJ-XSI)*Sigmay)*ET(K)/2.
      FSTAR(9)=0.
c     Transform element forces from local to global coordinates.
      CALL TRANSFMF(FSTAR,T,FT)
      OOBF(3*MI(K)-2)=OOBF(3*MI(K)-2)+FT(1)
      OOBF(3*MI(K)-1)=OOBF(3*MI(K)-1)+FT(2)
      OOBF(3*MI(K))=OOBF(3*MI(K))+FT(3)
      OOBF(3*MJ(K)-2)=OOBF(3*MJ(K)-2)+FT(4)
      OOBF(3*MJ(K)-1)=OOBF(3*MJ(K)-1)+FT(5)
      OOBF(3*MJ(K))=OOBF(3*MJ(K))+FT(6)
      OOBF(3*MK(K)-2)=OOBF(3*MK(K)-2)+FT(7)
      OOBF(3*MK(K)-1)=OOBF(3*MK(K)-1)+FT(8)
      OOBF(3*MK(K))=OOBF(3*MK(K))+FT(9)
c     Nodal forces due to {Ar} are included in {FUPDAT} and in {SUPDAT} which
c     are used above to generate {OOBF}. Deduct below the nodal forces due to
c     {SR}.
      Sigmax=SR(3*K-2)
      Sigmay=SR(3*K-1)
      Tau=SR(3*K)
c     Adjust the above stress-values to account for the change element size. The
c     adjusted stresses are equal to existing values multiplied by factors
c     determined below. For this purpose determine the initial size of element.
      XIIN=X(MI(K))
      YIIN=Y(MI(K))
      ZIIN=Z(MI(K))
      XJIN=X(MJ(K))
      YJIN=Y(MJ(K))
      ZJIN=Z(MJ(K))
      XKIN=X(MK(K))
      YKIN=Y(MK(K))
      ZKIN=Z(MK(K))
c     Direction cosines of local axes of element in its initial geometry.
      CALL COSLOCAL(XIIN,YIIN,ZIIN,XJIN,YJIN,ZJIN,XKIN,YKIN,ZKIN,
     +TIN,ALIJIN,ALRKIN,ALIRIN)
      Sigmax=Sigmax*ALRKIN/ALRK
      Sigmay=Sigmay*ALIJIN/ALIJ
      Tau=Tau*(ALRKIN*ALIJIN)/(ALRK*ALIJ)
      FSTAR(1)=(-(YSK-YSJ)*Sigmax-(XSJ-XSK)*Tau)*ET(K)
      FSTAR(2)=((XSK-XSJ)*Sigmay-(YSK-YSJ)*Tau)*ET(K)
      FSTAR(3)=0.
      FSTAR(4)=(-(YSI-YSK)*Sigmax-(XSK-XSI)*Tau)*ET(K)
      FSTAR(5)=((XSI-XSK)*Sigmay-(YSI-YSK)*Tau)*ET(K)
      FSTAR(6)=0.
      FSTAR(7)=(-(XSI-XSJ)*Tau)*ET(K)
      FSTAR(8)=((XSJ-XSI)*Sigmay)
      FSTAR(9)=0.
c     Transform element forces from local to global coordinates.
      CALL TRANSFMF(FSTAR,T,FT)
      OOBF(3*MI(K)-2)=OOBF(3*MI(K)-2)-FT(1)
      OOBF(3*MI(K)-1)=OOBF(3*MI(K)-1)-FT(2)
      OOBF(3*MI(K))=OOBF(3*MI(K))-FT(3)
      OOBF(3*MJ(K)-2)=OOBF(3*MJ(K)-2)-FT(4)
      OOBF(3*MJ(K)-1)=OOBF(3*MJ(K)-1)-FT(5)
      OOBF(3*MJ(K))=OOBF(3*MJ(K))-FT(6)
      OOBF(3*MK(K)-2)=OOBF(3*MK(K)-2)-FT(7)
      OOBF(3*MK(K)-1)=OOBF(3*MK(K)-1)-FT(8)
      OOBF(3*MK(K))=OOBF(3*MK(K))-FT(9)
11    CONTINUE
c     The reactions are calculated as the forces balancing the member end
c     forces (see below). Therefore, the out-of-balance force at a coord-
c     inates with prescribed displacement is set to zero.
      DO 30 I=1,NSJ
      IF(ISUP(I,1).EQ.0)OOBF(3*NS(I)-2)=0.
      IF(ISUP(I,2).EQ.0)OOBF(3*NS(I)-1)=0.
30    IF(ISUP(I,3).EQ.0)OOBF(3*NS(I))=0.
c     Sum the square of out-of-balance forces for assesment of error ratio.
      SUMOOBF=0.
      DO 20 I=1,NJ3
20    SUMOOBF=SUMOOBF+ OOBF(I)*OOBF(I)
      SUMOOBF=SQRT(SUMOOBF)
c     Calculate the updated reactions from the updated member end forces.
      DO 1 I=1,NSJ
      DO 1 J=1,3
1     R(I,J)=0.
      DO 16 L=1,NSJ
      DO 15 K=1,NBAR
      IF((NS(L).NE.JS(K)).AND.(NS(L).NE.JE(K)))GO TO 15
      ALX=XUPDAT(JE(K))-XUPDAT(JS(K))
      ALY=YUPDAT(JE(K))-YUPDAT(JS(K))
      ALZ=ZUPDAT(JE(K))-ZUPDAT(JS(K))
      AL=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      CX=ALX/AL
      CY=ALY/AL
      CZ=ALZ/AL
      A2=AUPDAT(K)
      A1=-A2
      IF(JS(K).EQ.NS(L))R(L,1)=R(L,1)+(A1*CX)
      IF(JS(K).EQ.NS(L))R(L,2)=R(L,2)+(A1*CY)
      IF(JS(K).EQ.NS(L))R(L,3)=R(L,3)+(A1*CZ)
      IF(JE(K).EQ.NS(L))R(L,1)=R(L,1)+(A2*CX)
      IF(JE(K).EQ.NS(L))R(L,2)=R(L,2)+(A2*CY)
      IF(JE(K).EQ.NS(L))R(L,3)=R(L,3)+(A2*CZ)
15    CONTINUE
16    CONTINUE
      DO 18 L=1,NSJ
      DO 17 K=1,NME
      IF((NS(L).NE.MI(K)).AND.(NS(L).NE.MJ(K)).AND.(NS(L).NE.MK(K)))
     +GO TO 17
      Sigmax=SUPDAT(3*K-2)
      Sigmay=SUPDAT(3*K-1)
      Tau=SUPDAT(3*K)
      XI=XUPDAT(MI(K))
      YI=YUPDAT(MI(K))
      ZI=ZUPDAT(MI(K))
      XJ=XUPDAT(MJ(K))
      YJ=YUPDAT(MJ(K))
      ZJ=ZUPDAT(MJ(K))
      XK=XUPDAT(MK(K))
      YK=YUPDAT(MK(K))
      ZK=ZUPDAT(MK(K))
c     Direction cosines of local axes of element
      CALL COSLOCAL(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,ALIJ,ALRK,ALIR)
c     Find coordinates of element corners with respect to local axes.
      CALL TRANSFCOR(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,
     + XSI,YSI,ZSI,XSJ,YSJ,ZSJ,XSK,YSK,ZSK)
c     Element nodal forces in local directions corresponding to stresses
c     {SUPDAT} (Eq. 24-72 of Ghali and Neville).
      FSTAR(1)=(-(YSK-YSJ)*Sigmax-(XSJ-XSK)*Tau)*ET(K)/2.
      FSTAR(2)=((XSK-XSJ)*Sigmay-(YSK-YSJ)*Tau)*ET(K)/2.
      FSTAR(3)=0.
      FSTAR(4)=(-(YSI-YSK)*Sigmax-(XSK-XSI)*Tau)*ET(K)/2.
      FSTAR(5)=((XSI-XSK)*Sigmay-(YSI-YSK)*Tau)*ET(K)/2.
      FSTAR(6)=0.
      FSTAR(7)=(-(XSI-XSJ)*Tau)*ET(K)/2.
      FSTAR(8)=((XSJ-XSI)*Sigmay)*ET(K)/2.
      FSTAR(9)=0.
c     Transform element forces from local to global coordinates.
      CALL TRANSFMF(FSTAR,T,FT)
      IF(MI(K).EQ.NS(L))R(L,1)=R(L,1)+FT(1)
      IF(MI(K).EQ.NS(L))R(L,2)=R(L,2)+FT(2)
      IF(MI(K).EQ.NS(L))R(L,3)=R(L,3)+FT(3)
      IF(MJ(K).EQ.NS(L))R(L,1)=R(L,1)+FT(4)
      IF(MJ(K).EQ.NS(L))R(L,2)=R(L,2)+FT(5)
      IF(MJ(K).EQ.NS(L))R(L,3)=R(L,3)+FT(6)
      IF(MK(K).EQ.NS(L))R(L,1)=R(L,1)+FT(7)
      IF(MK(K).EQ.NS(L))R(L,2)=R(L,2)+FT(8)
      IF(MK(K).EQ.NS(L))R(L,3)=R(L,3)+FT(9)
17    CONTINUE
18    CONTINUE
c     Determine sum of squares of the reaction components.
      SUMR=0.
      DO 21 I=1,NSJ
      DO 21 J=1,3
      IF(ISUP(I,J).EQ.0)SUMR=SUMR+R(I,J)*R(I,J)
21    CONTINUE
      SUMR=SQRT(SUMR)
c     The reactions calculated from member end forces do not include the effect
c     of external forces applied along a reaction component. Add the missing
c     values.
      DO 22 I=1,NSJ
      DO 22 J=1,3
      IF(ISUP(I,J).EQ.0)R(I,J)=R(I,J)+COOF*RSAVE(I,J)
22    CONTINUE
      RETURN
      END

      SUBROUTINE STIF(JS,JE,A,XUPDAT,YUPDAT,ZUPDAT,S,NJ3,NB,AUPDAT,
     + SUPDAT,X,Y,Z,ET,MI,MJ,MK,SL)
c     Tangent stiffness matrix for spatial structure composed of pin-connected
c     bars and triangular membrane elements.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION JS(NBAR),JE(NBAR),A(NBAR)
      DIMENSION XUPDAT(NJ),YUPDAT(NJ),ZUPDAT(NJ)
      DIMENSION S(NJ3,NB),AUPDAT(NBAR),SUPDAT(3*NME),X(NJ),Y(NJ),Z(NJ)
      DIMENSION MI(NME),MJ(NME),MK(NME),ET(NME),SL(NB-1,NJ3-1)
      DIMENSION ES(6,6),EST(9,9),IM(9),T(3,3),TT(3,3),AA(3,3),FSTAR(9)
      DIMENSION CC(3,3)
      DO 10 I=1,NJ3
      DO 10 J=1,NB
      S(I,J)=0.0
10    CONTINUE
      DO 1 I=1,NB-1
      DO 1 J=1,NJ3-1
      SL(I,J)=0.0
1     CONTINUE
      DO 50 JM=1,NBAR
      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALZ=Z(JE(JM))-Z(JS(JM))
      ALJINIT=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      ALX=XUPDAT(JE(JM))-XUPDAT(JS(JM))
      ALY=YUPDAT(JE(JM))-YUPDAT(JS(JM))
      ALZ=ZUPDAT(JE(JM))-ZUPDAT(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      CX=ALX/ALJ
      CY=ALY/ALJ
      CZ=ALZ/ALJ
      AJ=A(JM)
      Axial=AUPDAT(JM)
      ES(1,1)=(EB*AJ/ALJINIT)*CX*CX+(Axial/ALJ)*(1.-CX*CX)
      ES(2,1)=(EB*AJ/ALJINIT)*CY*CX+(Axial/ALJ)*(-CX*CY)
      ES(3,1)=(EB*AJ/ALJINIT)*CZ*CX+(Axial/ALJ)*(-CX*CZ)
      ES(2,2)=(EB*AJ/ALJINIT)*CY*CY+(Axial/ALJ)*(1.-CY*CY)
      ES(3,2)=(EB*AJ/ALJINIT)*CZ*CY+(Axial/ALJ)*(-CY*CZ)
      ES(3,3)=(EB*AJ/ALJINIT)*CZ*CZ+(Axial/ALJ)*(1.-CZ*CZ)
      ES(1,2)=ES(2,1)
      ES(1,3)=ES(3,1)
      ES(2,3)=ES(3,2)
      DO 11 I=4,6
      DO 11 J=4,6
11    ES(I,J)=ES(I-3,J-3)
      DO 12 I=4,6
      DO 12 J=1,3
12    ES(I,J)=-ES(I-3,J)
      DO 13 I=1,3
      DO 13 J=4,6
13    ES(I,J)=ES(J,I)
c     Identify the six coordinates at the start and end of bar.
      IM(1)=3*JS(JM)-2
      IM(2)=IM(1)+1
      IM(3)=IM(1)+2
      IM(4)=3*JE(JM)-2
      IM(5)=IM(4)+1
      IM(6)=IM(4)+2
      DO 40 I=1,6
      DO 40 J=1,6
      K=IM(J)-IM(I)
      IF(K.LT.0)GO TO 35
      K=K+1
      S(IM(I),K)=S(IM(I),K)+ES(I,J)
      GO TO 40
35    CONTINUE
      K=-K
      SL(K,IM(J))=SL(K,IM(J))+ES(I,J)
40    CONTINUE
50    CONTINUE
c     Stiffness of membrane elements
      d11=EM/(1.-PRM*PRM)
      d21=PRM*d11
      d22=d11
      d33=d11*(1.-prm)/2.
      DO 80 K=1,NME
      XI=XUPDAT(MI(K))
      YI=YUPDAT(MI(K))
      ZI=ZUPDAT(MI(K))
      XJ=XUPDAT(MJ(K))
      YJ=YUPDAT(MJ(K))
      ZJ=ZUPDAT(MJ(K))
      XK=XUPDAT(MK(K))
      YK=YUPDAT(MK(K))
      ZK=ZUPDAT(MK(K))
c     Direction cosines of local axes of element
      CALL COSLOCAL(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,ALIJ,ALRK,ALIR)
c     Find coordinates of element corners with respect to local axes.
      CALL TRANSFCOR(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,
     + XSI,YSI,ZSI,XSJ,YSJ,ZSJ,XSK,YSK,ZSK)
      ai=XSJ*YSK-YSJ*XSK
      bi=YSJ-YSK
      ci=XSK-XSJ
      aj=XSK*YSI-YSK*XSI
      bj=YSK-YSI
      cj=XSI-XSK
      ak=XSI*YSJ-YSI*XSJ
      bk=YSI-YSJ
      ck=XSJ-XSI
      DO 51 I=1,9
      DO 51 J=1,9
51    EST(I,J)=0.
      Area=(ALIJ*ALRK)/2.
c     Elastic stiffness matrix by Eq. 17-79 of Ghali and Neville.
      EST(1,1)=d11*bi*bi+d33*ci*ci
      EST(2,1)=d21*bi*ci+d33*bi*ci
      EST(2,2)=d22*ci*ci+d33*bi*bi
      EST(4,1)=d11*bi*bj+d33*ci*cj
      EST(4,2)=d21*ci*bj+d33*cj*bi
      EST(4,4)=d11*bj*bj+d33*cj*cj
      EST(5,1)=d21*bi*cj+d33*bj*ci
      EST(5,2)=d22*ci*cj+d33*bi*bj
      EST(5,4)=d21*bj*cj+d33*cj*bj
      EST(5,5)=d22*cj*cj+d33*bj*bj
      EST(7,1)=d11*bi*bk+d33*ci*ck
      EST(7,2)=d21*ci*bk+d33*bi*ck
      EST(7,4)=d11*bj*bk+d33*cj*ck
      EST(7,5)=d21*cj*bk+d33*bj*ck
      EST(7,7)=d11*bk*bk+d33*ck*ck
      EST(8,1)=d21*bi*ck+d33*bk*ci
      EST(8,2)=d22*ci*ck+d33*bi*bk
      EST(8,4)=d21*bj*ck+d33*cj*bk
      EST(8,5)=d22*cj*ck+d33*bj*bk
      EST(8,7)=d21*bk*ck+d33*ck*bk
      EST(8,8)=d22*ck*ck+d33*bk*bk
      DO 52 I=1,8
      DO 52 J=I+1,9
52    EST(I,J)=EST(J,I)
      CCC=ET(K)/(4*AREA)
      DO 53 I=1,9
      DO 53 J=1,9
53    EST(I,J)=EST(I,J)*CCC
c     Add geometric stiffness matrix (Eq. 24-79 of Ghali and Neville). For this
c     purpose, we need the nodal forces corresponding to the current stresses
c     in the element (Eq. 24-72)..
      Sigmax=SUPDAT(3*K-2)
      Sigmay=SUPDAT(3*K-1)
      Tau   =SUPDAT(3*K)
      FSTAR(1)=(-(YSK-YSJ)*Sigmax-(XSJ-XSK)*Tau)*ET(K)/2.
      FSTAR(2)=((XSK-XSJ)*Sigmay-(YSK-YSJ)*Tau)*ET(K)/2.
      FSTAR(3)=0.
      FSTAR(4)=(-(YSI-YSK)*Sigmax-(XSK-XSI)*Tau)*ET(K)/2.
      FSTAR(5)=((XSI-XSK)*Sigmay-(YSI-YSK)*Tau)*ET(K)/2.
      FSTAR(6)=0.
      FSTAR(7)=(-(XSI-XSJ)*Tau)*ET(K)/2.
      FSTAR(8)=((XSJ-XSI)*Sigmay)*ET(K)/2.
      FSTAR(9)=0.
c     Element rotations due to unit displacement in the z* direction(Eqs. 24-80,
c     24-81 and 24-82).
      tix=-(XSJ-XSK)/((XSJ-XSI)*(YSK-YSJ))
      tiy=1./(XSJ-XSI)
      tjx=-(XSK-XSI)/((XSJ-XSI)*(YSK-YSJ))
      tjy=-1./(XSJ-XSI)
      tkx=1./(YSK-YSJ)
      h=ET(K)
      EST(4,1)=EST(4,1)+h*Tau/2.
      EST(5,1)=EST(5,1)+h*Sigmay/2.
      EST(7,1)=EST(7,1)-h*Tau/2.
      EST(8,1)=EST(8,1)-h*Sigmay/2.
      EST(4,2)=EST(4,2)-h*Sigmax/2.
      EST(5,2)=EST(5,2)-h*Tau/2.
      EST(7,2)=EST(7,2)+h*Sigmax/2.
      EST(8,2)=EST(8,2)+h*Tau/2.
      EST(3,3)=EST(3,3)+FSTAR(2)*tix-FSTAR(1)*tiy
      EST(6,3)=EST(6,3)+FSTAR(5)*tix-FSTAR(4)*tiy
      EST(9,3)=EST(9,3)+FSTAR(8)*tix-FSTAR(7)*tiy
      EST(1,4)=EST(1,4)-h*Tau/2.
      EST(2,4)=EST(2,4)-h*Sigmay/2.
      EST(7,4)=EST(7,4)+h*Tau/2.
      EST(8,4)=EST(8,4)+h*Sigmay/2.
      EST(1,5)=EST(1,5)+h*Sigmax/2.
      EST(2,5)=EST(2,5)+h*Tau/2.
      EST(7,5)=EST(7,5)-h*Sigmax/2.
      EST(8,5)=EST(8,5)-h*Tau/2.
      EST(3,6)=EST(3,6)+FSTAR(2)*tjx-FSTAR(1)*tjy
      EST(6,6)=EST(6,6)+FSTAR(5)*tjx-FSTAR(4)*tjy
      EST(9,6)=EST(9,6)+FSTAR(8)*tjx-FSTAR(7)*tjy
      EST(1,7)=EST(1,7)+h*Tau/2.
      EST(2,7)=EST(2,7)+h*Sigmay/2.
      EST(4,7)=EST(4,7)-h*Tau/2.
      EST(5,7)=EST(5,7)-h*Sigmay/2.
      EST(1,8)=EST(1,8)-h*Sigmax/2.
      EST(2,8)=EST(2,8)-h*Tau/2.
      EST(4,8)=EST(4,8)+h*Sigmax/2.
      EST(5,8)=EST(5,8)+h*Tau/2.
      EST(3,9)=EST(3,9)+FSTAR(2)*tkx
      EST(6,9)=EST(6,9)+FSTAR(5)*tkx
      EST(9,9)=EST(9,9)+FSTAR(8)*tkx
c     Transform element stiffness to global coordinates.
      DO 62 I=1,3
      DO 62 J=1,3
62    TT(I,J)=T(J,I)
      DO 65 II=1,3
      DO 64 JJ=1,3
      DO 61 I=1,3
      DO 61 J=1,3
61    AA(I,J)=EST((II-1)*3+I,(JJ-1)*3+J)
      CALL MULTIPLY(TT,AA,CC,3,3,3)
      CALL MULTIPLY(CC,T,AA,3,3,3)
      DO 63 I=1,3
      DO 63 J=1,3
63    EST((II-1)*3+I,(JJ-1)*3+J)=AA(I,J)
64    CONTINUE
65    CONTINUE
c     Identify the nine coordinates at corners of triangular element.
      IM(1)=3*MI(K)-2
      IM(2)=IM(1)+1
      IM(3)=IM(1)+2
      IM(4)=3*MJ(K)-2
      IM(5)=IM(4)+1
      IM(6)=IM(4)+2
      IM(7)=3*MK(K)-2
      IM(8)=IM(7)+1
      IM(9)=IM(7)+2
      DO 70 I=1,9
      DO 70 J=1,9
      KK=IM(J)-IM(I)
      IF(KK.LT.0)SL(-KK,IM(J))=SL(-KK,IM(J))+EST(I,J)
      IF(KK.GE.0)S(IM(I),KK+1)=S(IM(I),KK+1)+EST(I,J)
70    CONTINUE
80    CONTINUE
      RETURN
      END

      SUBROUTINE MULTIPLY(A,B,C,N,M,KK)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N,M),B(M,KK),C(N,KK)
      DO 10 I=1,N
      DO 10 J=1,KK
      SUM=0
      DO 20 L=1,M
20    SUM=SUM+A(I,L)*B(L,J)
10    C(I,J)=SUM
      RETURN
      END

      SUBROUTINE SUPTPF(S,OOBF,NS,ISUP,PDUPDAT,NJ3,NB,ITER)
c     Adjust [S] and {OOBF} in accordance with the support conditions.See
c     Eq. 22-39 of Ghali and Neville.Note that before the adjustment,
c     [S] generated by the subroutine STIFPF is the stiffness matrix of
c     a free unsupported structure.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION S(NJ3,NB),OOBF(NJ3),NS(NSJ),ISUP(NSJ,3)
      DIMENSION PDUPDAT(NSJ,3)
      DO 50 I=1,NSJ
      N1=3*NS(I)-3
      DO 40 K=1,3
      IF(ISUP(I,K).EQ.1)GO TO 30
      N=N1+K
      SKM=S(N,1)
      S(N,1)=(S(N,1)+1.)*1.0E6
      IF(ITER.GT.1)GO TO 30
      OOBF(N)=-(SKM+1.)*1.0E6*PDUPDAT(I,K)
30    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE UPDATEA(D,XUPDAT,YUPDAT,ZUPDAT,JS,JE,A,AUPDAT,NJ3,
     + X,Y,Z,AINIT)
c     Update axial forces in bars by adding the effect of the displacement
c     increment {D}.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION D(NJ3),AINIT(NBAR),AUPDAT(NBAR)
      DIMENSION JS(NBAR),JE(NBAR),A(NBAR)
      DIMENSION XUPDAT(NJ),YUPDAT(NJ),ZUPDAT(NJ),X(NJ),Y(NJ),Z(NJ)
      DO 10 K=1,NBAR
c     Length of bar after latest displacement increment.
      ALX=XUPDAT(JE(K))+D(3*JE(K)-2)-(XUPDAT(JS(K))+D(3*JS(K)-2))
      ALY=YUPDAT(JE(K))+D(3*JE(K)-1)-(YUPDAT(JS(K))+D(3*JS(K)-1))
      ALZ=ZUPDAT(JE(K))+D(3*JE(K))-  (ZUPDAT(JS(K))+D(3*JS(K)))
      ALJ=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      ALJINIT=DSQRT( (X(JE(K))-X(JS(K))) *(X(JE(K))-X(JS(K)))
     +		    +(Y(JE(K))-Y(JS(K))) *(Y(JE(K))-Y(JS(K)))
     +		    +(Z(JE(K))-Z(JS(K))) *(Z(JE(K))-Z(JS(K)))  )
      DELTAL=ALJ-ALJINIT
      AUPDAT(K)=AINIT(K)+DELTAL*EB*A(K)/ALJINIT
10    CONTINUE
      RETURN
      END

      SUBROUTINE UPDATES(D,XUPDAT,YUPDAT,ZUPDAT,MI,MJ,MK,SUPDAT,NJ3,
     +	X,Y,Z,SINIT)
c     Update membrane stresses in elements by adding effects of displacement
c     increment {D}.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION D(NJ3),SUPDAT(3*NME),MI(NME),MJ(NME),MK(NME)
      DIMENSION XUPDAT(NJ),YUPDAT(NJ),ZUPDAT(NJ),X(NJ),Y(NJ),Z(NJ)
      DIMENSION SINIT(3*NME),T(3,3)
      DO 10 K=1,NME
c     Geometry of element after displacement increments {D}.
      XI=XUPDAT(MI(K))+D(3*MI(K)-2)
      YI=YUPDAT(MI(K))+D(3*MI(K)-1)
      ZI=ZUPDAT(MI(K))+D(3*MI(K))
      XJ=XUPDAT(MJ(K))+D(3*MJ(K)-2)
      YJ=YUPDAT(MJ(K))+D(3*MJ(K)-1)
      ZJ=ZUPDAT(MJ(K))+D(3*MJ(K))
      XK=XUPDAT(MK(K))+D(3*MK(K)-2)
      YK=YUPDAT(MK(K))+D(3*MK(K)-1)
      ZK=ZUPDAT(MK(K))+D(3*MK(K))
c     Direction cosines of local axes of element after displacement increment
      CALL COSLOCAL(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,ALIJ,ALRK,ALIR)
c     Initial element geometry.
      XI=X(MI(K))
      YI=Y(MI(K))
      ZI=Z(MI(K))
      XJ=X(MJ(K))
      YJ=Y(MJ(K))
      ZJ=Z(MJ(K))
      XK=X(MK(K))
      YK=Y(MK(K))
      ZK=Z(MK(K))
c     Initial lengths of element.
      CALL COSLOCAL(XI,YI,ZI,XJ,YJ,ZJ,XK,YK,ZK,T,ALIJin,ALRKin,ALIRin)
c     Changes in strains from initial to latest deformed state Eq. 24-83 of
c     Ghali and Neville 4th ed..
      Epsylonx=(ALIJ-ALIJin)/ALIJin
      Epsylony=(ALRK-ALRKin)/ALRKin
      Gamma=(1./ALRKin)*(ALIR-ALIRin-Epsylonx*ALIRin)
c     Updated element stresses (Eq. 24-84 of Ghali and Neville 4th ed. .
      SUPDAT(3*K-2)=(EM/(1.-PRM*PRM))*Epsylonx+
     +		PRM*(EM/(1.-PRM*PRM))*Epsylony+SINIT(3*K-2)*ALRKin/ALRK
      SUPDAT(3*K-1)=(EM/(1.-PRM*PRM))*Epsylony+
     +		PRM*(EM/(1.-PRM*PRM))*Epsylonx+SINIT(3*K-1)*ALIJin/ALIJ
      SUPDAT(3*K)=(EM/(2.*(1.+PRM)))*Gamma+
     +		SINIT(3*K)*ALIJin*ALRKin/(ALIJ*ALRK)
10    CONTINUE
      RETURN
      END

      SUBROUTINE UPDATED(DUPDAT,D,NJ3)
c     Add the increment in displacements to the existing values stored in
c     {DUPDAT}.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION D(NJ3),DUPDAT(NJ3)
      DO 10 I=1,NJ3
10    DUPDAT(I)=DUPDAT(I)+D(I)
      RETURN
      END

      SUBROUTINE RESULTS(DUPDAT,NS,R,AUPDAT,SUPDAT,NJ3,ITER,ERROR)
c     Printing of nodal displacement reactions and member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NBAR,NME,NSJ,EB,EM,PRM
      DIMENSION DUPDAT(NJ3),NS(NSJ),AUPDAT(NBAR),SUPDAT(3*NME)
      DIMENSION R(NSJ,3)
      WRITE(6,1000)
1000  FORMAT(3X,'------------------------------------------------------
     *-------------------------------------------------')
      WRITE(6,1100)
1100  FORMAT(3X,'Analysis results')
      WRITE(6,200)
200   FORMAT(/,3X,'Nodal displacements')
      WRITE(6,300)
300   FORMAT(5X,'Node',9X,'u',15X,'v',13x,'  w  ')
      DO 10 I=1,NJ
      N1=3*I-2
      N2=N1+1
      N3=N1+2
      WRITE(6,400)I,DUPDAT(N1),DUPDAT(N2),DUPDAT(N3)
400   FORMAT(3X,I5,3(1X,E16.8))
10    CONTINUE
      WRITE(6,500)
500   FORMAT(/,3X,'Forces at the supported nodes')
      WRITE(6,600)
600   FORMAT(5X,'NODE',8X,'Fx',14X,'Fy',14X,'Fz')
      DO 30 L=1,NSJ
      WRITE(6,400)NS(L),R(L,1),R(L,2),R(L,3)
30    CONTINUE
      WRITE(6,700)
700   FORMAT(/,3X,'Axial forces in bars')
      WRITE(6,800)
800   FORMAT(5X,'Bar   ', 8X,'Axial force')
      DO 50 JM=1,NBAR
50    WRITE(6,900)JM,AUPDAT(JM)
900   FORMAT(3X,I5,9X,E12.5)
      WRITE(6,750)
750   FORMAT(/,3X,'Stresses in membrane elements')
      WRITE(6,850)
850   FORMAT(3X,'Element',4X,'Sigmax',7X,'Sigmay',10X,'Tauxy')
      DO 55 JM=1,NME
55    WRITE(6,950)JM,SUPDAT(3*JM-2),SUPDAT(3*JM-1),SUPDAT(3*JM)
950   FORMAT(3X,I5,3(2X,E12.5))
      WRITE(6,1250)(ITER-1),ERROR
1250  FORMAT(/,3X,'Number of iterations performed in this stage=',I2,/,
     +3X,'Error=',E12.5)
      WRITE(6,1200)
1200  FORMAT(3X,'-------------------------------------------------------
     *------------------------------------------------')
60    CONTINUE
      RETURN
      END

c------------------------------------------------------------------------------
c		   Solution of simultaneous equations
c		   Based on Ghali,A. and Neville,A. M.,
c      Structural Analysis :A Unified Classical and Matrix Approach
c      Chapman and Hall,London and New York,3 ed.,1990,Section A-9

c     CALL SOLVE(S,SL,NB,D,OOBF,NJ3,1)
      SUBROUTINE SOLVE(A,AL,NB,C,B,N,M)
c     Solution of equilibrium equations [S][D]=-[F], using Gauss elemination.
c     The stiffness matrix is nonsymmetrical. [A] stands for [S] and [AL]
c     stands for [SL]. [B] stands for [-F]; [C] stands for [D]
c     The matrix [A] has band width= NB. Thus the elements of [A] are stored
c     in [S]n*NB for elements on and above the diagonal and in matrix
c     [SL](NB-1)*(n-1), for elements below the diagonal.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N,NB),AL(NB-1,N-1),B(N,M),C(N,M)
      DO 1 I=1,N
      DO 1 J=1,M
      B(I,J)=-B(I,J)
1     C(I,J)=0.
c     Generation of auxiliary matrix and store it in [A] and [B]
c     First row of auxiliary matrix
      IF(A(1,1).LE.0.)WRITE(6,100)
100   FORMAT(/,3X,'Zero or negative value encountered on the diagonal',
     +/,3X,'of the stiffness matrix at row 1.')
      DO 10 J=2,NB
10    A(1,J)=A(1,J)/A(1,1)
      DO 20 J=1,M
20    B(1,J)=B(1,J)/A(1,1)
c     Remaining rows of auxiliary matrix
      DO 31 I=2,N
      DO 30 J=2,N
      IF(J.GT.I)GO TO 36
      IF(J.EQ.I)GO TO 366
      IF(I.GT.(NB+J-1))GO TO 38
      SUM=0.
      DO 35 K=1,J-1
35    IF((J.LE.(NB+K-1)).AND.((I-K).LE.(NB-1)))
     +	       SUM=SUM+AL(I-K,K)*A(K,J-K+1)
      AL(I-J,J)=AL(I-J,J)-SUM
      GO TO 38
366   SUM=0.
      DO 355 K=1,I-1
355   IF(J.LE.(NB+K-1))SUM=SUM+AL(I-K,K)*A(K,J-K+1)
      A(I,J-I+1)=A(I,J-I+1)-SUM
      IF(A(I,1).LE.0.)WRITE(6,200)I
200   FORMAT(/,3X,'Zero or negative value encountered on the diagonal',
     +/,3X,'of the stiffness matrix or in the Gauss elimination, while'
     +,/,3X,'solving the equilibrium equations on row',I4)
      GO TO 38
36    IF(J.GT.(NB+I-1))GO TO 38
      SUM=0.
      DO 37 K=1,I-1
37    IF((I.LE.(NB+K-1)).AND.((J-K+1).LE.NB))
     +	      SUM=SUM+AL(I-K,K)*A(K,J-K+1)
      A(I,J-I+1)=(A(I,J-I+1)-SUM)/A(I,1)
38    CONTINUE
30    CONTINUE
31    CONTINUE
      DO 40 I=2,N
      DO 40 J=1,M
      SUM=0
      DO 45 K=1,I-1
45    IF((I-K).LE.(NB-1))SUM=SUM+AL(I-K,K)*B(K,J)
      B(I,J)=(B(I,J)-SUM)/A(I,1)
40    CONTINUE
c     Back substitution; store solution in [C]
      DO 50 L=1,M
      C(N,L)=B(N,L)
      DO 50 I=2,N
      IB=N-I+1
      SUM=0
      DO 55 K=IB+1,N
      IF(K.LE.(NB+IB-1))SUM=SUM+A(IB,K-IB+1)*C(K,L)
55    CONTINUE
50    C(IB,L)=B(IB,L)-SUM
      RETURN
      END
