c			 Computer Program PLASTICF
c                      Plastic Analysis of Plane Frames  
c                             January 2016

c			     by A. Ghali
c		       Professor of Civil Engineering
c			  The University of Calgary
c			   Calgary, Canada T2N 1N4

c  Basis: Ghali,A. and Neville,A.M.,Structural Analysis:A Unified Classical
c   and Matrix Approach, CRC Press, Spon Press, 7th ed., 2016, Chapter 18.
c------------------------------------------------------------------------------
c				 Scope
c    PLASTICF analyses plane frames with straight prismatic members. Axial and
c  bending deformations are considered; deformations due to shear are ignored.
c  In input file, all nodal forces and member fixed-end forces are expressed
c  in terms of an unknown multiplier,SUMINC. The analysis gives the value of
c  the multiplier just sufficient to produce collapse. It also gives
c  location and sequence of development of plastic hinges, the corresponding
c  displacements and member end forces.

c    The loading can be any, or cmbination,of the following types:
c  (a) Two concentrated forces and a couple,Fx,Fy and Mz at a node.
c  (b) General load or temperature variation in a member; in which case the
c      six member fixed-end forces, {Ar} must be included in the input.

c   The analysis can also be for the effect of prescribed nodal displacement
c  combined with these two load types. Only one load combination can be
c  treated in one run.

c    Plastic hinges are assumed to occur only at member ends, when the
c  absolute value of member end moments reach given plastic moment values, Mp. Thus, when
c  a plastic hinge is expected to occur at a particular section, e.g. below a
c  a concentrated load, a node should be selected at this location
c  (subdivision of member).

c			      Sign Convention
c     Positive directions of nodal displacements u, v and theta and  member
c  end forces are defined in Figs. 22-1 and 22-2 of Ghali and Neville. The
c  reaction components are determined in the directions of the global axes.
c
c			  Input Data Instructions
c     The data, in free format, must be in a file named PLASTICF.IN. The input
c  file is is composed of single lines or sets of lines as follows:
c  - Single line of title ,not exceeding 72 characters.
c  - Single line of 3 integers: number of nodes, number of members, number of
c    joints with prescribed displacement(s).
c  - Single line of one real value for the elasticity modulus, E.
c  - A set of lines; their number equals number of nodes. Each line is
c    composed of a node number followed by its x and y coordinates.
c  - A set of lines their number equals number of members. Each line starts
c    with 3 integers: a member number, its starting node and its end node,
c    followed by three real values: a, I and Mp.
c  - A set of lines their number equals number of joints with prescribed
c    displacement. Each line starts with a node number and three restraint
c    indicators (1 or 0),followed by three real values of prescribed disp-
c    lacements. The restraint indicators 1 and 0 mean free and prescribed
c    displacement component, respectively. When a displacement component is
c    free, a dummy value must be filled in for the prescribed displacement
c    (any real number).
c  - A set of any number of lines for loading of type(a). Each line is comp-
c    osed of one integer: a node number, followed by three real values of
c    nodal forces Fx, Fy and Mz. The last line of the set must start by an
c    integer greater than the number of load cases follwed three  dummy real
c    numbers. The last line must not be omitted even when there is no loading
c    of type (a).
c  - A set of any number of lines for loading of type(b).Each line is comp-
c    osed of one integers: a member number, followed by six real values of
c    member end forces. The last line of this set must start with an integer
c    greater than the number of load cases followed by six dummy real values.
c    The last line must not be omitted even when there is no loading of
c    type (b).

c    Examle input data:

c Plane frame of Example 24-3, 7th edition of Ghali and Neville.
c  8	7    2			       NJ,NM,NSJ
c  4175e3			       Elasticity modulus

c  1	     0.0	26.0	       Node number,x and y coordinates
c  2	     0.0	16.0
c  3	     0.0	 6.0
c  4	    10.0	 3.0
c  5	    20.0	 0.0
c  6	    40.0	 6.0
c  7	    50.0	 9.0
c  8	    50.0	26.0

c  1	1    2	 0.2044  0.1440    765.0    Member, JS, JE, a, I, Mp
c  2	2    3	 0.2044  0.1440    765.0
c  3	3    4	 0.3266  0.2464   1275.0
c  4	4    5	 0.3266  0.2464   1275.0
c  5	5    6	 0.2653  0.1934   1015.0
c  6	6    7	 0.2653  0.1934   1015.0
c  7	7    8	 0.1716  0.1140    616.0

c  1	0   0	0   0.0   0.0	 0.0   Node no.,restr. indics.,prsc. displs.
c  8	0   0	0   0.0   0.0	 0.0

c  2	1.0    0.0    0.0	       Node number, Fx, Fy, Mz
c  4	0.0    4.0    0.0
c  6	0.0    3.0    0.0
c 100	3*0.0	Dummy, end of data of loaded nodes(integer>NJ,Fx,Fy,Mz)

c 100	6*0.0	Dummy, end of data of member loads(integer>NM,{Ar})

c All loads and prescribed displacements are in terms of the unknown collapse
c multiplier, SUMINC.

c   When it is desired to ignore axial deformatios(e.g. to compare the results
c  with hand solution), enter large values for a. Note that using an
c  excessively large number for the parameter a may produce numerical
c  difficulty in solution of the equilibrium equations, giving wrong
c  displacements. To run the program, type PLASTICF and press Enter. The
c  input and the results will be in a file named PLASTICF.OUT.

c			     Limits on Data
c  Vectors {B} and {NI} are used to store all the real and the integer arrays
c  used by the program. The maximum dimensions set are: MAXB=20000 and MAXNI=
c  2000 . This would normally be sufficient for a frame approximately 120 nodes,
c  120 members, depending upon the band width. The maximum dimensions may be
c  changed as required on line 2 of program(see listing below). After the
c  change, the program and EQSOLN.FOR (equation solution subroutine) must be
c  compiled and linked using a Fortran compiler(e.g. Microsoft Fortran
c  Optimizing compiler). The actual storage space required for the given
c  problem will be printed in the output. If the required space exceeds the
c  available space, the output will indicate that MAXB or MAXNI, or both, need
c  to be increased.

c			 Collapse Criterion
c     The collapse is considered reached when one of the following occurs:
c  (a) The ratio Dnew/Dold is greater than 100.0; where DOLD and DNEW are
c     length of displacement vectors due to the preceding and the current
c     load increments.
c  (b) The stiffness matrix becomes not positive definite.
c     When any of the two conditions occurs the program stops and the collapse
c     load is given by the last printed value of SUMINC.

c------------------------------------------------------------------------------
c		     Notation and dimensions of arrays
c								     rows  cols.
c  {A}	  cross-section areas					     NM
c  {AI}   second moments of areas of cross-sections		     NM
c  [AR]   member-end-forces with nodal displacements restrained      6	   NRAR
c	  (given data)
c  {AU}   calculated member-end-forces with load multiplier=1.0      3*NM
c  {D}	  nodal displacements, when the load multiplier=1.0	     3*NJ
c  E	  modulus of elasticity
c {IPH}   plastic hinge indicators: integer 0, 1, 2 or 3 ; 0 refers  NM
c	  to no plastic hinges; 1 refers to hinge at starting end;
c	  2 refers to hinge at the other end; 3 refers to hinges
c	  at both ends.
c  [ES]   element (member) stiffness in local coordinates	     6	    6
c  [EST]  element stiffness matrix in directions parallel to	     6	    6
c	  global axes
c  {F}	  nodal restraining forces, due to forces applied at nodes.  3*NJ
c	  This vector is sustained for all load increments.
c  {Fadjust} nodal restraining forces due forces applied at the      3*NJ
c	  nodes combined with loading of type (b). This vector is
c	  adjusted during the solution of the equilibrium equation.
c	  Thus, the vector is recalculated for each load increment
c	  using adjusted values of {Ar} for members with one or
c	  more plastic hinges.
c  [ISUP] joint restraint indicators: 1 and 0, indicating free	     NSJ    3
c	  and prescribed displacement respectively
c  {JE}   end nodes of individual members			     NM
c  {JS}   starting node of individual members			     NM
c  LINC   load increment number
c  {LR}   list of member numbers for which fixed-end forces are      NRAR
c	  given as data
c  NB	  band width ; see Sec. 22-3 of Ghali and Neville
c  NJ	  number of joints (nodes)
c  NM	  number of members
c {NN}	  vector composed of two integers representing the two ends  2*NM
c	  for each member; the integer is either zero or the number
c	  of the member node,if a plastic hinge is introduced.
c  NRAR   number of columns in [AR]; this is equal the number of mem-
c	  bers for which fixed-end forces are given as data
c  {NS}   list of node numbers where supports are provided or disp-  NSJ
c	  lacements are prescribed
c  NSJ	  number of supprted joints (nodes with prescribed displac-
c	  ements)
c  [PD]   prescribed displacement values			     NSJ   3
c  {PM}   plastic moment capacity formembers			     NM
c  [R]	  forces at the supprted nodes including the reactions,      3*NSJ NLC
c	  with the load multiplier=1.0
c  [S]	  stiffness matrix					     3*NJ   NB
c  {SK}   diagonal elements of [S] corresponding to  coordinates     3*NSJ
c	  at the supportd nodes
c  [SUMA] sum of member-end-force increments			     6*NM
c  {SUMD} sum of nodal displacements incements			     3*NJ
c  SUMINC sum of load increment multipliers
c  {SUMR} sum of reaction increments
c  VINC   value of load increment multiplier
c  {X}	  x coordinate of nodes 				     NJ
c  {Y}	  y coordinates of nodes				     NJ
c------------------------------------------------------------------------------

c		    Listing of program PLASTICF
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXB=20000,MAXNI=2000)
      COMMON/REALV/B(MAXB)
      COMMON/NINTGR/NI(MAXNI)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',ACCESS='SEQUENTIAL',FILE='PLASTICF.IN')
      OPEN(6,FORM='FORMATTED',ACCESS='SEQUENTIAL',FILE='PLASTICF.OUT')

      DO 10 I=1,MAXB
10    B(I)=0.0
      DO 25 I=1,MAXNI
25    NI(I)=0
      READ(5,100)TITLE
100   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(3X,A76)
      READ(5,*)NJ,NM,NSJ
      WRITE(6,300)NJ
300   FORMAT(3X,'Number of joints=',I5)
      WRITE(6,400)NM
400   FORMAT(3X,'Number of members=',I5)
      WRITE(6,500)NSJ
500   FORMAT(3X,'Number of joints with prescribed displacement(s)=',I5)
      NJ3=3*NJ
      NS3=3*NSJ
      NM6=6*NM

c     Elements of {B} at which real arrays start are:
c     {A}:N1 {AI}:N2   [D]:N3  [F]:N4  [PD]:N5	[R]:N6	{SK}:N7
c     {X}:N8  {Y}:N9  {PM}:N10	 [S]:N11  [SUMD]:N12  [SUMR]:N13
c     [SUMA]:N14  [AU]:N15 {Fadjust}:N16  [AR]:N17

      N1=1
      N2=N1+NM
      N3=N2+NM
      N4=N3+NJ3
      N5=N4+NJ3
      N6=N5+NS3
      N7=N6+NS3
      N8=N7+NS3
      N9=N8+NJ
      N10=N9+NJ
      N11=N10+NM

c     Elements of {NI} at which integer arrays start are:
c     {JS}:K1  {JE}:K2	{NS}:K3  [ISUP]:K4  {IPH}:K5 {NN}:K6 [LR]:K7

      K1=1
      K2=K1+NM
      K3=K2+NM
      K4=K3+NSJ
      K5=K4+NS3
      K6=K5+NM
      K7=K6+2*NM

c     CALL DATAPF(X,Y,JS,JE,A,AI,NS,ISUP,PD,NB,PM)
      CALL DATAPF(B(N8),B(N9),NI(K1),NI(K2),B(N1),B(N2),NI(K3),
     *NI(K4),B(N5),NB,B(N10))

c     Identification of elements of {B} at which real arrays start
c     (continued)

      N12=N11+NJ3*NB
      N13=N12+NJ3
      N14=N13+NS3
      N15=N14+NM6
      N16=N15+NM6
      N17=N16+NJ3

c     Determine maximum number of rows of [AR] that can be stored in
c     the remaining space in {B}. Also determine the maximum number of
c     rows of [LR] which can be stored in the remaining space in {NI}.
      NAR=(MAXB-N17)/6
      NLR=(MAXNI-K7)

c     CALL LOADPF(F,LR,AR,NJ3,NLR,NAR)
      CALL LOADPF(B(N4),NI(K7),B(N17),NJ3,
     *NLR,NAR)

      WRITE(6,15)MAXB,(N17+6*NRAR-1)
15    FORMAT(/,3X,'Storage space of real numbers',/,3X,
     +'Maximum available:',I6,3X,';required storage:',I6)
      WRITE(6,16)MAXNI,(K7+NRAR-1)
16    FORMAT(/,3X,'Storage space of integers',/,3X,
     +'Maximum available:',I6,3X,';required storage:',I7)
      IF((N17+6*NRAR-1).GT.MAXB)WRITE(6,17)
17    FORMAT(/,3X,
     +'Increase MAXB (second FORTRAN statement); then compile.')
      IF((K7+NRAR-1).GT.MAXNI)WRITE(6,18)
18    FORMAT(/,3X,
     +'Increase MAXNI (second FORTRAN statement); then compile.')
      IF((N17+6*NRAR-1).GT.MAXB)GO TO 999
      IF((K7+NRAR-1).GT.MAXNI)GO TO 999

c     Apply load increments .

      LINC=0
      SUMINC=0.0

20    CONTINUE
      LINC=LINC+1

c     CALL ARADJUST(AR,IPH,NAR,LR,NLR,X,Y,JS,JE,Fadjust,F)
      CALL ARADJUST(B(N17),NI(K5),NAR,NI(K7),NLR,B(N8),B(N9),
     +NI(K1),NI(K2),B(N16),B(N4))

c     CALL STIFPF(JS,JE,A,AI,X,Y,S,NJ3,NB,IPH)
      CALL STIFPF(NI(K1),NI(K2),B(N1),B(N2),B(N8),B(N9),B(N11),
     *NJ3,NB,NI(K5))

c     CALL SUPTPF(S,Fadjust,NS,ISUP,PD,R,SK,NJ3,NS3,NB)
      CALL SUPTPF(B(N11),B(N16),NI(K3),NI(K4),B(N5),B(N6),B(N7),NJ3,NS3,
     *NB)

c     CALL UNITLOAD(S,Fadjust,D,NS,ISUP,PD,SK,X,Y,JS,JE,A,AI,LR,
c    *AR,NLR,NAR,NJ3,NS3,R,AU,IPH,NB)
      CALL UNITLOAD(B(N11),B(N16),B(N3),NI(K3),NI(K4),B(N5),B(N7),B(N8),
     +B(N9),NI(K1),NI(K2),B(N1),B(N2),NI(K7),B(N17),NLR,NAR,NJ3,NS3,
     +B(N6),B(N15),NI(K5),NB)

c     CALL INCREMENT(AU,SUMA,PM,VINC)
      CALL INCREMENT(B(N15),B(N14),B(N10),VINC)
      SUMINC=SUMINC+VINC

c     CALL DINCREMENT(VINC,D,SUMD,DOLD,DNEW)
      CALL DINCREMENT(VINC,B(N3),B(N12),DOLD,DNEW)

c     DOLD and DNEW are length of displacement vectors due to the preceding and
c     the current load increments. Consider that the cllapse occurs when the
c     ratio (DNEW/DOLD) is greater than 100.0 .
      IF(LINC.EQ.1)GO TO 99
      IF((DNEW/DOLD).GT.100.0)GO TO 999

99    CONTINUE
      WRITE(6,1000)
1000  FORMAT(3X,'------------------------------------------------------
     *-------------------------------------------------')
      WRITE(6,1100)
1100  FORMAT(3X,'Analysis results')

c     CALL RESULTS(D,NS,R,AU,
c    *NJ3,NS3,SUMD,SUMR,SUMA,LINC,VINC,SUMINC,IPH,
c    +PM,NN,JS,JE)
      CALL RESULTS(B(N3),NI(K3),B(N6),B(N15),
     +NJ3,NS3,B(N12),B(N13),B(N14),LINC,VINC,SUMINC,NI(K5),
     +B(N10),NI(K6),NI(K1),NI(K2))

      GOTO 20
999   CLOSE(5)
      CLOSE(6)
      STOP
      END
c     ***************************************************************

      SUBROUTINE DATAPF(X,Y,JS,JE,A,AI,NS,ISUP,PD,NB,PM)
c     Reading data for plane frame (excluding loads)and calculation of
c     band width.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION X(NJ),Y(NJ),JS(NM),JE(NM),A(NM),AI(NM)
      DIMENSION NS(NSJ),ISUP(NSJ,3),PD(NSJ,3),PM(NM)
      READ(5,*)E
      WRITE(6,200)E
200   FORMAT(3X,'Elasticity modulus=',E13.5,2X)
      WRITE(6,300)
300   FORMAT(/,3X,'Nodal coordinates')
      WRITE(6,400)
400   FORMAT(5X,'Node',6x,'x',9X,'y')
      DO 10 I=1,NJ
      READ(5,*)J,X(J),Y(J)
10    WRITE(6,500)J,X(J),Y(J)
500   FORMAT(3X,I5,2F10.3)
      WRITE(6,600)
600   FORMAT(/,3X,'Element information')
      WRITE(6,700)
700   FORMAT(/,3X,'Element',2X,'1st node',2X,'2nd node',5X,'a',10X,'I',
     *10X,'Mp')
      DO 20 I=1,NM
      READ(5,*)J,JS(J),JE(J),A(J),AI(J),PM(J)
c     Member no.,starting node,end node,cross-section area,2nd moment
c     of area and plastic moment.
20    WRITE(6,800)J,JS(J),JE(J),A(J),AI(J),PM(J)
800   FORMAT(3X,I5,4X,I5,4X,I5,4X,4(E10.3,1X))
      WRITE(6,900)
900   FORMAT(/,3X,'Support conditions')
      WRITE(6,1000)
1000  FORMAT(5X,'Node',4X,'Restraint indicators',7X,'Prescribed
     *displacements')
c     Restraint indicators are 1 or o,meaning free or prescribed disp-
c     lacement,respectively.
      WRITE(6,1100)
1100  FORMAT(16x,'u',6x,'v',3X,'theta',8x,'u',13x,'v',12x,'theta')
1200  FORMAT(3X,I5,4X,I5,2X,I5,2X,I5,3(2X,E13.5))
      DO 30 J=1,NSJ
      READ(5,*)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),PD(J,
     *3)
30    WRITE(6,1200)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),
     *PD(J,3)
c     Determine band width by Eq. 24-1 of Ghali and Neville.
      NBI=0
      DO 50 I=1,NM
      N=JS(I)-JE(I)
      IF(N.GE.0) GO TO 40
      N=-N
40    CONTINUE
      IF(N.GT.NBI)NBI=N
50    CONTINUE
      NB=3*(NBI+1)
      RETURN
      END

      SUBROUTINE STIFPF(JS,JE,A,AI,X,Y,S,NJ3,NB,IPH)
c     Stiffness matrix for plane frame.See Sec. 22-9,Ghali and Neville
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION JS(NM),JE(NM),A(NM),AI(NM),X(NJ),Y(NJ)
      DIMENSION S(NJ3,NB)
      DIMENSION ES(6,6),EST(6,6),IM(6),IPH(NM)
      N3=3*NJ
      DO 10 I=1,N3
      DO 10 J=1,NB
      S(I,J)=0.0
10    CONTINUE
      DO 50 JM=1,NM
      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/ALJ
      SIN=ALY/ALJ
      AJ=A(JM)
      AIJ=AI(JM)
      IPHJ=IPH(JM)
c     Member stiffness matrix in local coordinates

      CALL ESTIFF(ALJ,AJ,AIJ,E,ES,IPHJ)

c     Transform element stiffness,[ES] into [EST] in global directions;
c     see Eq. 22-27 of Ghali and Neville.
c     transformation had to be adapted to a more general case
      DO 20 I=1,6
      DO 20 J=1,6
20    EST(I,J)=0.0
      EST(1,1)=ES(1,1)*C*C+ES(2,2)*SIN*SIN
      EST(4,4)=ES(4,4)*C*C+ES(5,5)*SIN*SIN
      EST(4,1)=ES(4,1)*C*C-ES(4,2)*SIN*C+ES(5,2)*SIN*SIN
      EST(3,3)=ES(3,3)
      EST(6,6)=ES(6,6)
      EST(2,2)=ES(1,1)*SIN*SIN+ES(2,2)*C*C
      EST(5,5)=ES(4,4)*SIN*SIN+ES(5,5)*C*C
      EST(5,2)=ES(4,1)*SIN*SIN+ES(4,2)*SIN*C+ES(5,2)*C*C
      EST(6,3)=ES(6,3)
      EST(2,1)=ES(1,1)*SIN*C-ES(2,2)*C*SIN
      EST(5,1)=ES(4,1)*SIN*C-ES(5,2)*SIN*C-ES(4,2)*SIN*SIN
      EST(5,4)=ES(4,4)*SIN*C-ES(5,5)*SIN*C
      EST(3,1)=-ES(3,2)*SIN
      EST(4,3)=ES(4,3)*C-ES(5,3)*SIN
      EST(6,1)=-ES(6,2)*SIN
      EST(6,4)=-ES(6,5)*SIN
      EST(3,2)=ES(3,2)*C
      EST(5,3)=ES(5,3)*C+ES(4,3)*SIN
      EST(4,2)=ES(4,1)*SIN*C+ES(4,2)*C*C-ES(5,2)*SIN*C
      EST(6,2)=ES(6,2)*C
      EST(6,5)=ES(6,5)*C
      DO 30 I=1,6
	  DO 30 J=I+1,6
30	  EST(I,J)=EST(J,I)

c     Identify the six coordinates at the start and end of member.
      IM(1)=3*JS(JM)-2
      IM(2)=IM(1)+1
      IM(3)=IM(1)+2
      IM(4)=3*JE(JM)-2
      IM(5)=IM(4)+1
      IM(6)=IM(4)+2
      DO 40 I=1,6
      DO 40 J=1,6
      K=IM(J)-IM(I)
      IF(K.LT.0)GO TO 35
      K=K+1
      S(IM(I),K)=S(IM(I),K)+EST(I,J)
35    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END


      SUBROUTINE ESTIFF(ALJ,AJ,AIJ,E,ES,IPHJ)
c     Element stiffness matrix in local coordinates; member of plane
c     frame with or without plastic hinges.
c     Shear deformation is neglected.
c     ALJ=member length; AJ=a; AIJ=I.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION ES(6,6)
      DO 10 I=1,6
      DO 10 J=1,6
10    ES(I,J)=0.0
      ES(1,1)=E*AJ/ALJ
      ES(4,1)=-ES(1,1)
      ES(4,4)=ES(1,1)
      IF(IPHJ.EQ.0)ES(2,2)=12.0*E*AIJ/(ALJ*ALJ*ALJ)
      IF(IPHJ.EQ.1)ES(2,2)=3.0*E*AIJ/(ALJ*ALJ*ALJ)
      IF(IPHJ.EQ.2)ES(2,2)=3.0*E*AIJ/(ALJ*ALJ*ALJ)
      IF(IPHJ.EQ.3)ES(2,2)=0.0
      ES(5,2)=-ES(2,2)
      ES(5,5)=ES(2,2)
      IF(IPHJ.EQ.0)ES(3,3)=4.0*E*AIJ/ALJ
      IF(IPHJ.EQ.1)ES(3,3)=0.0
      IF(IPHJ.EQ.2)ES(3,3)=3.0*E*AIJ/ALJ
      IF(IPHJ.EQ.3)ES(3,3)=0.0
      IF(IPHJ.EQ.0)ES(6,6)=ES(3,3)
      IF(IPHJ.EQ.1)ES(6,6)=3.0*E*AIJ/ALJ
      IF(IPHJ.EQ.2)ES(6,6)=0.0
      IF(IPHJ.EQ.3)ES(3,3)=0.0
      IF(IPHJ.EQ.0)ES(3,2)=ES(2,2)*ALJ/2.0
      IF(IPHJ.EQ.1)ES(3,2)=0.0
      IF(IPHJ.EQ.2)ES(3,2)=3.0*E*AIJ/(ALJ*ALJ)
      IF(IPHJ.EQ.3)ES(3,2)=0.0
      IF(IPHJ.EQ.0)ES(6,2)=ES(3,2)
      IF(IPHJ.EQ.1)ES(6,2)=3.0*E*AIJ/(ALJ*ALJ)
      IF(IPHJ.EQ.2)ES(6,2)=0.0
      IF(IPHJ.EQ.3)ES(6,2)=0.0
      IF(IPHJ.EQ.0)ES(5,3)=-ES(6,2)
      IF(IPHJ.EQ.1)ES(5,3)=0.0
      IF(IPHJ.EQ.2)ES(5,3)=-3.0*E*AIJ/(ALJ*ALJ)
      IF(IPHJ.EQ.3)ES(5,3)=0.0
      IF(IPHJ.EQ.0)ES(6,3)=2.0*E*AIJ/ALJ
      IF(IPHJ.EQ.1)ES(6,3)=0.0
      IF(IPHJ.EQ.2)ES(6,3)=0.0
      IF(IPHJ.EQ.3)ES(6,3)=0.0
      IF(IPHJ.EQ.0)ES(6,5)=-ES(3,2)
      IF(IPHJ.EQ.1)ES(6,5)=-3.0*E*AIJ/(ALJ*ALJ)
      IF(IPHJ.EQ.2)ES(6,5)=0.0
      IF(IPHJ.EQ.3)ES(6,5)=0.0
      DO 20 I=1,6
      DO 20 J=I+1,6
      ES(I,J)=ES(J,I)
20    CONTINUE
      RETURN
      END

      SUBROUTINE LOADPF(F,LR,AR,NJ3,NLR,NAR)
c     Generation of vectors of restraining forces,[F]
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION F(NJ3),LR(NLR)
      DIMENSION AR(6,NAR)
c     Forces applied at the nodes.Note that the last line of this set
c     of input lines must start with an integer > number of load cases;
c     the line must be completed by another integer and 3 real numbers.
      WRITE(6,100)
100   FORMAT(/,3X,'Multiples of collapse load applied at the nodes')
      WRITE(6,200)
200   FORMAT(3X,'Node',7X,'Fx',12X,'Fy',12X,'Mz')
20    CONTINUE
      READ(5,*)NODE,FX,FY,AMZ
      WRITE(6,300)NODE,FX,FY,AMZ
300   FORMAT(3X,I5,3(3X,E11.5))
      IF(NODE.GT.NJ)GO TO 30
      N1=3*NODE-2
      N2=N1+1
      N3=N2+1
      F(N1)=F(N1)-FX
      F(N2)=F(N2)-FY
      F(N3)=F(N3)-AMZ
      GO TO 20
30    CONTINUE
c     Fixed-end forces(given in data)due to loading away from the nodes
c     or due to temperature,etc..Note that the last line in this set of
c     data lines must start with an integer > number of load cases;the
c     line must be completed by another integer and six real numbers.
c     The positive directions of the end forces are shown in Fig.22-2 of
c     Ghali and Neville, 4th edition.
      WRITE(6,400)
400   FORMAT(/,3X,'Member end forces with nodal displacement restrained
     *')
      WRITE(6,500)
500   FORMAT(3X,'Member',2X,'Ar1',8X,'Ar2',8X,'Ar3',
     *8X,'Ar4',8X,'Ar5',8X,'Ar6')
      NRAR=0
c     NRAR is number of columns in [AR]; that is equal to the number of
c     lines for loading of type (b).The last line of data is not inc-
c     luded.
40    CONTINUE
      NRAR=NRAR+1
      N=NRAR
      READ(5,*)LR(N),AR(1,N),AR(2,N),AR(3,N),AR(4,N),AR(5,N),
     *AR(6,N)
      WRITE(6,600)LR(N),AR(1,N),AR(2,N),AR(3,N),AR(4,N),
     *AR(5,N),AR(6,N)
600   FORMAT(3X,I5,1X,5(E10.4,1x),E10.4)
      IF(LR(N).GT.NJ)GO TO 50
      GO TO 40
50    CONTINUE
      NRAR=NRAR-1
      RETURN
      END

      SUBROUTINE SUPTPF(S,Fadjust,NS,ISUP,PD,R,SK,NJ3,NS3,NB)
c     Adjust [S] and [F] in accordance with the support conditions.See
c     Eq. 24-39 of Ghali and Neville.Note that before the adjustment,
c     [S] generated by the subroutine STIFPF is the stiffness matrix of
c     a free unsupported structure.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION S(NJ3,NB),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION R(NS3),SK(NS3),Fadjust(NJ3)
c     The adjusted force vector is stored in {Fadjust} in order to maintain {F}
c     unchanged to be used in analysis for all load increments.

      DO 10 I=1,NS3
      SK(I)=0.0
10    R(I)=0.0
c     Use {SK} and [R] to store appropriate elements on the diagonal of
c     [S] and of [F] for use in calculation of the reactions by Eq. 22-
c     38 of Ghali and Neville.
      DO 50 I=1,NSJ
      M1=3*I-3
      N1=3*NS(I)-3
      DO 40 K=1,3
      IF(ISUP(I,K).EQ.1)GO TO 30
      M=M1+K
      N=N1+K
      SK(M)=S(N,1)+1
      S(N,1)=S(N,1)+1
      S(N,1)=S(N,1)*1.0E6
      R(M)=Fadjust(N)
      Fadjust(N)=-SK(M)*1.0E6*PD(I,K)
30    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE HINGELOCAT(SUMA,PM,IPH,NN,JS,JE)
c     Identify the location of plastic hinges by integers 0,1,2 or 3 stored in
c     {IPH}. 0 refers to a member without plastic hinges; 1 refers to one hinge
c     at the starting end; 2 refers to hinge at the other end; 3 refers to a
c     hinge at each end .
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION SUMA(6*NM),PM(NM),IPH(NM),NN(2*NM),JS(NM),JE(NM)
      DO 10 I=1,NM
      AA=PM(I)-ABS(SUMA(6*(I-1)+3) )
      BB=PM(I)-ABS(SUMA(6*(I-1)+6) )
      IF(   (ABS(AA).GT.(PM(I)*1.e-6)).AND.
     +	    (ABS(BB).GT.(PM(I)*1.e-6)) )IPH(I)=0
      IF(   (ABS(AA).LE.(PM(I)*1.e-6)).AND.
     +	    (ABS(BB).GT.(PM(I)*1.e-6)) )IPH(I)=1
      IF(   (ABS(AA).GT.(PM(I)*1.e-6)).AND.
     +	    (ABS(BB).LE.(PM(I)*1.e-6)) )IPH(I)=2
      IF(   (ABS(AA).LE.(PM(I)*1.e-6)).AND.
     +	    (ABS(BB).LE.(PM(I)*1.e-6)) )IPH(I)=3
10    CONTINUE

c     The plastic hinges must not be introduced at all member ends meeting at
c     an interior joint, because this will make the joint unstable. Adjust {IPH}
c     to ensure that at interior joints, at least one member end has no plastic
c     hinge.
c     Generate {NN} composrd of two integers representing the two ends for each
c     member; the integer is either zero or the number of the node if a plastic
c     hinge is introduced.
      DO 20 I=1,NM
      IF(IPH(I).EQ.1)NN(2*(I-1)+1)=JS(I)
      IF(IPH(I).EQ.2)NN(2*(I-1)+2)=JE(I)
      IF(IPH(I).EQ.3)NN(2*(I-1)+1)=JS(I)
      IF(IPH(I).EQ.3)NN(2*(I-1)+2)=JE(I)
20    CONTINUE

c     If a node number appears more than once in {NN}, keep the first, but
c     change the others to zero.
      DO 30 J=1,2*NM
      DO 25 K=J+1,2*NM
      IF(NN(J).NE.NN(K))GO TO 23
      NH=0
      DO 24 KK=J+1,2*NM
24    IF(NN(J).EQ.NN(KK))NH=NH+1
c     Count the number of members meeting at the node NN(J).
      IC=0
      DO 29 L=1,NM
      IF(JS(L).EQ.NN(J))IC=IC+1
29    IF(JE(L).EQ.NN(J))IC=IC+1
      IF((IC-NH).GT.1)GO TO 23
      NN(K)=0
23    CONTINUE
25    CONTINUE
30    CONTINUE

c     Adjust {IPH} to represent the plastic hinges retained.
      DO 40 I=1,NM
      IF((NN(2*(I-1)+1).EQ.0).AND.(NN(2*(I-1)+2).EQ.0))IPH(I)=0
      IF((NN(2*(I-1)+1).NE.0).AND.(NN(2*(I-1)+2).EQ.0))IPH(I)=1
      IF((NN(2*(I-1)+1).EQ.0).AND.(NN(2*(I-1)+2).NE.0))IPH(I)=2
      IF((NN(2*(I-1)+1).NE.0).AND.(NN(2*(I-1)+2).NE.0))IPH(I)=3
40    CONTINUE

      RETURN
      END

      SUBROUTINE ARADJUST(AR,IPH,NAR,LR,NLR,X,Y,JS,JE,Fadjust,F)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION AR(6,NAR),IPH(NM),LR(NLR)
      DIMENSION X(NJ),Y(NJ),JS(NM),JE(NM),Fadjust(3*NJ),F(3*NJ)
c     Generate a load vector to be used in calculation of the effect of a
c     unit load increment. First, consider loading type (a).
      DO 1 I=1,3*NJ
1     Fadjust(I)=F(I)

c     Adjust the fixed-end-moments using Eq. 11.46 for members with plastic
c     hinges. Also adjust the transverse end forces to maintain equilibrium.

      DO 10 J=1,NRAR
      JM=LR(J)
      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY)
      IF(IPH(JM).EQ.0)GO TO 7
      IF(IPH(JM).EQ.1)AR(6,J)=AR(6,J)-AR(3,J)/2.
      IF(IPH(JM).EQ.1)AR(2,J)=AR(2,J)-1.5*AR(3,J)/ALJ
      IF(IPH(JM).EQ.1)AR(5,J)=AR(5,J)+1.5*AR(3,J)/ALJ
      IF(IPH(JM).EQ.1)AR(3,J)=0.
      IF(IPH(JM).EQ.1)GO TO 7
      IF(IPH(JM).EQ.2)AR(3,J)=AR(3,J)-AR(6,J)/2.
      IF(IPH(JM).EQ.2)AR(2,J)=AR(2,J)-1.5*AR(6,J)/ALJ
      IF(IPH(JM).EQ.2)AR(5,J)=AR(5,J)+1.5*AR(6,J)/ALJ
      IF(IPH(JM).EQ.2)AR(6,J)=0.
      IF(IPH(JM).EQ.2)GO TO 7
      IF((IPH(JM).EQ.3).AND.(AR(3,J).EQ.0.))GO TO 5
      IF((IPH(JM).EQ.3).AND.(AR(6,J).EQ.0.))GO TO 6
5     CONTINUE
      IF(AR(6,J).EQ.0.)GO TO 7
      AR(2,J)=AR(2,J)-AR(6,J)/ALJ
      AR(5,J)=AR(5,J)+AR(6,J)/ALJ
      AR(6,J)=0.
      GO TO 7
6     CONTINUE
      IF(AR(3,J).EQ.0.)GO TO 7
      AR(2,J)=AR(2,J)-AR(3,J)/ALJ
      AR(5,J)=AR(5,J)+AR(3,J)/ALJ
      AR(3,J)=0.
7     CONTINUE

c     Transform the fixed-end forces into global directions and generate {FAR}.
c     {FAR}=nodal forces due forces applied away from the nodes.To do this,iden-
c     tify the node and coordinate numbers at the member ends and calculate the
c     cosine and sine the angle that the member makes with the global
c     x axis.
      N1=3*JS(JM)-2
      N2=N1+1
      N3=N1+2
      C=ALX/ALJ
      SIN=ALY/ALJ
      Fadjust(N1)=Fadjust(N1)+AR(1,J)*C-AR(2,J)*SIN
      Fadjust(N2)=Fadjust(N2)+AR(1,J)*SIN+AR(2,J)*C
      Fadjust(N3)=Fadjust(N3)+AR(3,J)
      N1=3*JE(JM)-2
      N2=N1+1
      N3=N1+2
      Fadjust(N1)=Fadjust(N1)+AR(4,J)*C-AR(5,J)*SIN
      Fadjust(N2)=Fadjust(N2)+AR(4,J)*SIN+AR(5,J)*C
      Fadjust(N3)=Fadjust(N3)+AR(6,J)

10    CONTINUE

      RETURN
      END

      SUBROUTINE INCREMENT(AU,SUMA,PM,VINC)
c     Determine the load multiplier to the given loads to be used in the
c     current load increment.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION AU(6*NM),SUMA(6*NM),PM(NM)

      V=0.
      DO 30 I=1,NM
      IF( (PM(I)-ABS(SUMA(6*(I-1)+3))).LT.(PM(I)*1.e-6) )GO TO 6
      ALPHA=ABS(AU(6*(I-1)+3))/ (PM(I)-ABS(SUMA(6*(I-1)+3)))
      IF(ALPHA.GT.V)V=ALPHA
6     CONTINUE
      IF( (PM(I)-ABS(SUMA(6*(I-1)+6))).LT.(PM(I)*1.e-6) )GO TO 7
      ALPHA=ABS(AU(6*(I-1)+6))/ (PM(I)-ABS(SUMA(6*(I-1)+6)))
      IF(ALPHA.GT.V)V=ALPHA
7     CONTINUE
30    CONTINUE
      VINC=1./V

      RETURN
      END

      SUBROUTINE DINCREMENT(VINC,D,SUMD,DOLD,DNEW)
c     Prepare the parameters DOLD and DNEW (defined below) for use as croterion
c     for reaching collapse.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION D(3*NJ),SUMD(3*NJ)

c     DOLD= square root of the sum of SUMD(I) squared; where SUMD(I) are the
c     displacements due to all	preceding load increments.
c     DNEW= square root of the sum of the displacements due to the current load increment(VINC*D(I)) squared; where D(I) are the
c     (VINC*D(I)) squared.
      DOLD=0.0
      DO 5 I=1,3*NJ
5     DOLD=DOLD+SUMD(I)*SUMD(I)
      DOLD=DSQRT(DOLD)

      DO 10 I=1,3*NJ
      D(I)=VINC*D(I)
10    CONTINUE

      DNEW=0.0
      DO 25 I=1,3*NJ
25    DNEW=DNEW+D(I)*D(I)
      DNEW=DSQRT(DNEW)

      RETURN
      END

      SUBROUTINE UNITLOAD(S,Fadjust,D,NS,ISUP,PD,SK,X,Y,JS,JE,A,AI,LR,
     *AR,NLR,NAR,NJ3,NS3,R,AU,IPH,NB)
c     Printing of nodal displacement reactions and member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION S(NJ3,NB),Fadjust(NJ3)
      DIMENSION D(NJ3),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION R(NS3),AU(6*NM)
      DIMENSION SK(NS3),X(NJ),Y(NJ),JS(NM),JE(NM),A(NM),AI(NM)
      DIMENSION LR(NLR),AR(6,NAR),IPH(NM)
      DIMENSION ES(6,6)

      DO 1 I=1,NJ3
1     D(I)=0.
      DO 3 I=1,(6*NM)
3     AU(I)=0.

      CALL EQSOLN(S,Fadjust,D,NJ3,NB,1)

      DO 30 L=1,NSJ
      M1=3*L-3
      N1=3*NS(L)-3
      DO 20 J=1,3
      IF(ISUP(L,J).EQ.1)GO TO 20
      M=M1+J
      N=N1+J
      R(M)=R(M)+SK(M)*1.0E6*(PD(L,J)-D(N))+(SK(M)-1)*D(N)
20    CONTINUE
30    CONTINUE

      DO 50 JM=1,NM
      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/ALJ
      SIN=ALY/ALJ
      AJ=A(JM)
      AIJ=AI(JM)
      IPHJ=IPH(JM)
      CALL ESTIFF(ALJ,AJ,AIJ,E,ES,IPHJ)
      N1=3*JS(JM)-2
      N2=N1+1
      N3=N1+2
      N4=3*JE(JM)-2
      N5=N4+1
      N6=N4+2
c     Transform displacements at member ends into local coordinates.
      DT1=D(N1)*C+D(N2)*SIN
      DT2=-D(N1)*SIN+D(N2)*C
      DT3=D(N3)
      DT4=D(N4)*C+D(N5)*SIN
      DT5=-D(N4)*SIN+D(N5)*C
      DT6=D(N6)
c     For each member,calculate the product of its stiffness matrix by
c     its displacement vector.
      A1=ES(1,1)*DT1+ES(1,4)*DT4
      A2=ES(2,2)*DT2+ES(2,3)*DT3+ES(2,5)*DT5+ES(2,6)*DT6
      A3=ES(3,2)*DT2+ES(3,3)*DT3+ES(3,5)*DT5+ES(3,6)*DT6
      A4=ES(4,1)*DT1+ES(4,4)*DT4
      A5=ES(5,2)*DT2+ES(5,3)*DT3+ES(5,5)*DT5+ES(5,6)*DT6
      A6=ES(6,2)*DT2+ES(6,3)*DT3+ES(6,5)*DT5+ES(6,6)*DT6
c     Add the fixed-end forces included in the input data.
      IF(NRAR.EQ.0)GO TO 45
      DO 40 I=1,NRAR
      IF(LR(I).NE.JM) GO TO 35
      A1=A1+AR(1,I)
      A2=A2+AR(2,I)
      A3=A3+AR(3,I)
      A4=A4+AR(4,I)
      A5=A5+AR(5,I)
      A6=A6+AR(6,I)
35    CONTINUE
40    CONTINUE
45    CONTINUE
      AU(6*(JM-1)+1)=A1
      AU(6*(JM-1)+2)=A2
      AU(6*(JM-1)+3)=A3
      AU(6*(JM-1)+4)=A4
      AU(6*(JM-1)+5)=A5
      AU(6*(JM-1)+6)=A6
50    CONTINUE

      RETURN
      END

      SUBROUTINE RESULTS(D,NS,R,AU,
     *NJ3,NS3,SUMD,SUMR,SUMA,LINC,VINC,SUMINC,IPH,
     +PM,NN,JS,JE)
c     Printing of nodal displacement reactions and member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION D(NJ3),NS(NSJ),R(NS3),AU(6*NM)
      DIMENSION SUMR(NS3),SUMD(NJ3),SUMA(6*NM)
      DIMENSION IPH(NM),NN(2*NM),JS(NM),JE(NM),PM(NM)

      WRITE(6,100)LINC
100   FORMAT(/,3X,'Load increment number: ',I2)
      WRITE(6,110)VINC,SUMINC
110   FORMAT(/,3X,'Current load-multiplier increment=',E13.5,/,3X,
     +'Sum of multiplier increments including the current one=',E13.5)
      WRITE(6,200)

200   FORMAT(/,3X,'Nodal displacements after current increment')
      DO 5 I=1,NJ3
5     SUMD(I)=SUMD(I)+D(I)
      WRITE(6,300)
300   FORMAT(5X,'Node',9X,'u',15X,'v',13x,'theta')
      DO 10 I=1,NJ
      N1=3*I-2
      N2=N1+1
      N3=N1+2
      WRITE(6,400)I,SUMD(N1),SUMD(N2),SUMD(N3)
400   FORMAT(3X,I5,3(3X,E13.5))
10    CONTINUE

      WRITE(6,500)
500   FORMAT(/,3X,'Forces at the supported nodes')
      WRITE(6,600)
600   FORMAT(5X,'NODE',8X,'Fx',14X,'Fy',14X,'Mz')
20    CONTINUE
      DO 30 L=1,3*NSJ
30    SUMR(L)=SUMR(L)+VINC*R(L)
      DO 40 L=1,NSJ
      WRITE(6,400)NS(L),SUMR(3*(L-1)+1),SUMR(3*(L-1)+2),SUMR(3*(L-1)+3)
40    CONTINUE

      WRITE(6,700)
700   FORMAT(/,3X,'Member end forces')
      WRITE(6,800)
800   FORMAT(3X,'Member',7X,'F1',10X,'F2',10x,'F3',10x,'F4',10X,'F5',
     *10X,'F6')
      DO 45 I=1,6*NM
45    SUMA(I)=SUMA(I)+VINC*AU(I)

      DO 55 JM=1,NM
      WRITE(6,900)JM,SUMA(6*(JM-1)+1),
     +		     SUMA(6*(JM-1)+2),
     +		     SUMA(6*(JM-1)+3),
     +		     SUMA(6*(JM-1)+4),
     +		     SUMA(6*(JM-1)+5),
     +		     SUMA(6*(JM-1)+6)
55    CONTINUE
900   FORMAT(3X,I5,6(1X,E11.5))

      CALL HINGELOCAT(SUMA,PM,IPH,NN,JS,JE)
      WRITE(6,950)
950   FORMAT(/,3X,'Locations of plastic hinges',//,3X,
     +'Member      Starting node        End node')
      Nhinges=0
      DO 65 I=1,NM
      IF(IPH(I).EQ.0)WRITE(6,951)I
951   FORMAT(3X,I5,10X,'No',18X,'No')
      IF(IPH(I).EQ.1)WRITE(6,952)I
952   FORMAT(3X,I5,10X,'Yes',17X,'No')
      IF(IPH(I).EQ.2)WRITE(6,953)I
953   FORMAT(3X,I5,10X,'No',18X,'Yes')
      IF(IPH(I).EQ.3)WRITE(6,954)I
954   FORMAT(3X,I5,10X,'Yes',17X,'Yes')
      IF(IPH(I).EQ.1)Nhinges=Nhinges+1
      IF(IPH(I).EQ.2)Nhinges=Nhinges+1
      IF(IPH(I).EQ.3)Nhinges=Nhinges+2
65    CONTINUE
      WRITE(6,955)Nhinges
955   FORMAT(/,3X,'Total number of plastic hinges=',I2)
      WRITE(6,1200)
1200  FORMAT(3X,'-------------------------------------------------------
     *------------------------------------------------')
      RETURN
      END

c-----------------------------------------------------------------------
      SUBROUTINE EQSOLN(S,F,D,NDF,NB,NLC)
c     Solution of equilibrium equations [S][D]=-[F], using Cholesky's
c     method; see Sec. 21-11 of Ghali et al.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S[HUGE](NDF,NB),F(NDF,NLC),D(NDF,NLC)
      IF(S(1,1).LE. 0.0)GO TO 98
      DO 50 J=2,NDF
      JJ=J-1
      JJJ=J-NB+1
      IF(JJJ.LT.1)JJJ=1
      IF(JJ.EQ.1)GO TO 30
      DO 20 I=2,JJ
      II=I-1
      IF(II.LT.JJJ)GO TO 20
      SUM=S(I,J-I+1)
      DO 10 K=JJJ,II
10    SUM=SUM-S(K,I-K+1)*S(K,J-K+1)
      S(I,J-I+1)=SUM
20    CONTINUE
30    SUM=S(J,1)
      DO 40 K=JJJ,JJ
      T=S(K,J-K+1)/S(K,1)
      SUM=SUM-T*S(K,J-K+1)
40    S(K,J-K+1)=T
c     IF(SUM.LE.0.0)GO TO  98
      if(sum.lt.0.0)write(6,101)j
      if(sum.eq.0.0)go to 98
50    S(J,1)=SUM
c     At this stage,the first column of the band matrix [S] is replaced
c     by e11,e22,... and the remaining elements of the same matrix are
c     replaced by  elements uij using Eq. 21-49 to 21-53 of Ghali and
c     Neville. Now by forward substitution (Eq. 21-48), determine [g]
c     and store in [D].
      DO 95 L=1,NLC
      DO 70 I=1,NDF
      J=I-NB+1
      IF(I.LE.NB)J=1
      SUM=-F(I,L)
      KK=I-1
      IF(J.GT.KK)GO TO 65
      DO 60 K=J,KK
60    SUM=SUM-S(K,I-K+1)*D(K,L)
65    D(I,L)=SUM
70    CONTINUE
      DO 80 I=1,NDF
80    D(I,L)=D(I,L)/S(I,1)
c     By Eq. 21-46 of Ghali et al and backward substitution(Eq.21-
c     47),determine the required displacement values and store in [D].
      DO 95 II=1,NDF
      I=NDF-II+1
      J=I+NB-1
      IF(J.GT.NDF) J=NDF
      SUM=D(I,L)
      KK=I+1
      IF(KK.GT.J) GO TO 90
      DO 85 K=KK,J
85    SUM=SUM-S(I,K-I+1)*D(K,L)
90    D(I,L)=SUM
95    CONTINUE
      GO TO 99
98    CONTINUE
      WRITE(6,102)J
102   FORMAT(/,1X,
     +'Stiffness matrix is not positive definite. Zero value encountered
     +',/,1X,
     +'on the diagonal in solution of equations on row:',I4)
101   format(/,1X,
     +'Stiffness matrix is not positive definite. Negative value encount
     +ered',/,1X,
     +'on the diagonal in solution of equations on row:',I4)
      STOP
99    CONTINUE
      RETURN
      END

